@@ -5,32 +5,22 @@ from functools import partial
 from multiprocessing import set_start_method
 from typing import Any, Callable, List, Optional, Tuple, Union
 
-from deap import tools
-
-from fedot.core.composer.advisor import PipelineChangeAdvisor
 from fedot.core.composer.cache import OperationsCache
 from fedot.core.composer.composer import Composer, ComposerRequirements
-from fedot.core.composer.gp_composer.specific_operators import boosting_mutation, parameter_change_mutation
 from fedot.core.data.data import InputData
 from fedot.core.data.data_split import train_test_data_setup
 from fedot.core.data.multi_modal import MultiModalData
 from fedot.core.log import Log, default_log
-from fedot.core.optimisers.adapters import PipelineAdapter
-from fedot.core.optimisers.gp_comp.gp_optimiser import GPGraphOptimiser, GPGraphOptimiserParameters, \
-    GraphGenerationParams
-from fedot.core.optimisers.gp_comp.operators.inheritance import GeneticSchemeTypesEnum
-from fedot.core.optimisers.gp_comp.operators.mutation import MutationStrengthEnum, single_add_mutation, \
-    single_change_mutation, single_drop_mutation, single_edge_mutation, MutationTypesEnum
-from fedot.core.optimisers.gp_comp.operators.regularization import RegularizationTypesEnum
-from fedot.core.optimisers.gp_comp.param_free_gp_optimiser import GPGraphParameterFreeOptimiser
+from fedot.core.optimisers.gp_comp.operators.mutation import MutationStrengthEnum
+from fedot.core.optimisers.graph import OptGraph
 from fedot.core.pipelines.pipeline import Pipeline
-from fedot.core.pipelines.validation import validate, ts_rules, common_rules
-from fedot.core.repository.operation_types_repository import OperationTypesRepository, get_operations_for_task
-from fedot.core.repository.quality_metrics_repository import (ClassificationMetricsEnum, MetricsEnum,
-                                                              MetricsRepository, RegressionMetricsEnum)
-from fedot.core.repository.tasks import Task, TaskTypesEnum
+from fedot.core.pipelines.validation import common_rules, ts_rules, validate
+from fedot.core.repository.quality_metrics_repository import (MetricsEnum,
+                                                              MetricsRepository)
+from fedot.core.repository.tasks import TaskTypesEnum
 from fedot.core.validation.compose.tabular import table_metric_calculation
 from fedot.core.validation.compose.time_series import ts_metric_calculation
+from fedot.remote.remote_evaluator import RemoteEvaluator, init_data_for_remote_execution
 
 sample_split_ratio_for_tasks = {
     TaskTypesEnum.classification: 0.8,
@@ -46,7 +36,7 @@ def set_multiprocess_start_method():
 
 
 @dataclass
-class GPComposerRequirements(ComposerRequirements):
+class PipelineComposerRequirements(ComposerRequirements):
     """
     Dataclass is for defining the requirements for composition process of genetic programming composer
 
@@ -70,22 +60,22 @@ class GPComposerRequirements(ComposerRequirements):
 class GPComposer(Composer):
     """
     Genetic programming based composer
-    :param optimiser: optimiser generated in GPComposerBuilder
+    :param optimiser: optimiser generated in ComposerBuilder
     :param metrics: metrics used to define the quality of found solution.
     :param composer_requirements: requirements for composition process
-    :param initial_pipeline: defines the initial state of the population. If None then initial population is random.
+    :param initial_pipelines: defines the initial state of the population. If None then initial population is random.
     """
 
     def __init__(self, optimiser=None,
-                 composer_requirements: Optional[GPComposerRequirements] = None,
+                 composer_requirements: Optional[PipelineComposerRequirements] = None,
                  metrics: Union[List[MetricsEnum], MetricsEnum] = None,
-                 initial_pipeline: Optional[Pipeline] = None,
+                 initial_pipelines: Optional[List[Pipeline]] = None,
                  logger: Log = None):
 
         super().__init__(metrics=metrics, composer_requirements=composer_requirements,
-                         initial_pipeline=initial_pipeline)
+                         initial_pipelines=initial_pipelines)
 
-        self.cache = OperationsCache()
+        self.cache = OperationsCache(log=logger)
 
         self.optimiser = optimiser
         self.cache_path = None
@@ -111,7 +101,7 @@ class GPComposer(Composer):
 
         self.optimiser.graph_generation_params.advisor.task = data.task
 
-        if data.task == TaskTypesEnum.ts_forecasting:
+        if data.task.task_type == TaskTypesEnum.ts_forecasting:
             self.optimiser.graph_generation_params.rules_for_constraint = ts_rules + common_rules
         else:
             self.optimiser.graph_generation_params.rules_for_constraint = common_rules
@@ -131,16 +121,22 @@ class GPComposer(Composer):
             self.log.info("Hold out validation for graph composing was applied.")
             split_ratio = sample_split_ratio_for_tasks[data.task.task_type]
             train_data, test_data = train_test_data_setup(data, split_ratio)
+
+            if RemoteEvaluator().use_remote:
+                init_data_for_remote_execution(train_data)
+
             objective_function_for_pipeline = partial(self.composer_metric, self.metrics, train_data, test_data)
 
         if self.cache_path is None:
             self.cache.clear()
         else:
             self.cache.clear(tmp_only=True)
-            self.cache = OperationsCache(self.cache_path, clear_exiting=not self.use_existing_cache)
+            self.cache = OperationsCache(log=self.log, db_path=self.cache_path,
+                                         clear_exiting=not self.use_existing_cache)
 
-        best_pipeline = self.optimiser.optimise(objective_function_for_pipeline,
-                                                on_next_iteration_callback=on_next_iteration_callback)
+        opt_result = self.optimiser.optimise(objective_function_for_pipeline,
+                                             on_next_iteration_callback=on_next_iteration_callback)
+        best_pipeline = self._convert_opt_results_to_pipeline(opt_result)
 
         self.log.info('GP composition finished')
         self.cache.clear()
@@ -148,6 +144,11 @@ class GPComposer(Composer):
             self.tune_pipeline(best_pipeline, data, self.composer_requirements.timeout)
         return best_pipeline
 
+    def _convert_opt_results_to_pipeline(self, opt_result: Union[OptGraph, List[OptGraph]]) -> Pipeline:
+        return [self.optimiser.graph_generation_params.adapter.restore(graph)
+                for graph in opt_result] if isinstance(opt_result, list) \
+            else self.optimiser.graph_generation_params.adapter.restore(opt_result)
+
     def _cv_validation_metric_build(self, data):
         """ Prepare function for metric evaluation based on task """
         if isinstance(data, MultiModalData):
@@ -159,18 +160,21 @@ class GPComposer(Composer):
             if self.composer_requirements.validation_blocks is None:
                 self.log.info('For ts cross validation validation_blocks number was changed from None to 3 blocks')
                 self.composer_requirements.validation_blocks = 3
+
             metric_function_for_nodes = partial(ts_metric_calculation, data,
                                                 self.composer_requirements.cv_folds,
                                                 self.composer_requirements.validation_blocks,
                                                 self.metrics,
                                                 log=self.log)
+
         else:
             self.log.info("KFolds cross validation for pipeline composing was applied.")
+
             metric_function_for_nodes = partial(table_metric_calculation, data,
                                                 self.composer_requirements.cv_folds,
                                                 self.metrics,
-                                                log=self.log)
-
+                                                log=self.log,
+                                                cache=self.cache)
         return metric_function_for_nodes
 
     def composer_metric(self, metrics,
@@ -185,17 +189,14 @@ class GPComposer(Composer):
                 metrics = [metrics]
 
             if self.cache is not None:
-                # TODO improve cache
                 pipeline.fit_from_cache(self.cache)
 
-            if not pipeline.is_fitted:
-                self.log.debug(f'Pipeline {pipeline.root_node.descriptive_id} fit started')
-                pipeline.fit(input_data=train_data,
-                             time_constraint=self.composer_requirements.max_pipeline_fit_time)
-                try:
-                    self.cache.save_pipeline(pipeline)
-                except Exception as ex:
-                    self.log.info(f'Cache can not be saved: {ex}. Continue.')
+            self.log.debug(f'Pipeline {pipeline.root_node.descriptive_id} fit started')
+
+            pipeline.fit(input_data=train_data,
+                         time_constraint=self.composer_requirements.max_pipeline_fit_time,
+                         use_fitted=self.cache is not None)
+            self.cache.save_pipeline(pipeline)
 
             evaluated_metrics = ()
             for metric in metrics:
@@ -213,7 +214,6 @@ class GPComposer(Composer):
         except Exception as ex:
             self.log.info(f'Pipeline assessment warning: {ex}. Continue.')
             evaluated_metrics = None
-
         return evaluated_metrics
 
     @staticmethod
@@ -223,103 +223,3 @@ class GPComposer(Composer):
     @property
     def history(self):
         return self.optimiser.history
-
-
-class GPComposerBuilder:
-    def __init__(self, task: Task):
-        self._composer = GPComposer()
-        self.optimiser_parameters = GPGraphOptimiserParameters()
-        self.task = task
-        self.set_default_composer_params()
-
-    def can_be_secondary_requirement(self, operation):
-        models_repo = OperationTypesRepository()
-        data_operations_repo = OperationTypesRepository(operation_type='data_operation')
-
-        operation_name = models_repo.operation_info_by_id(operation)
-        if operation_name is None:
-            operation_name = data_operations_repo.operation_info_by_id(operation)
-        operation_tags = operation_name.tags
-
-        secondary_model = True
-        # TODO remove 'data_model'
-        if 'data_model' in operation_tags:
-            secondary_model = False
-        return secondary_model
-
-    def with_optimiser_parameters(self, optimiser_parameters: GPGraphOptimiserParameters):
-        self.optimiser_parameters = optimiser_parameters
-        return self
-
-    def with_requirements(self, requirements: GPComposerRequirements):
-        # TODO move this functionality in composer
-        requirements.secondary = list(filter(self.can_be_secondary_requirement, requirements.secondary))
-        self._composer.composer_requirements = requirements
-        return self
-
-    def with_metrics(self, metrics: Union[List[MetricsEnum], MetricsEnum]):
-        if type(metrics) is not list:
-            metrics = [metrics]
-        self._composer.metrics = metrics
-        return self
-
-    def with_initial_pipeline(self, initial_pipeline: Optional[Pipeline]):
-        self._composer.initial_pipeline = initial_pipeline
-        return self
-
-    def with_logger(self, logger):
-        self._composer.log = logger
-        return self
-
-    def with_cache(self, cache_path: str = None, use_existing=False):
-        self._composer.cache_path = cache_path
-        self._composer.use_existing_cache = use_existing
-        return self
-
-    def set_default_composer_params(self):
-        """ Method set metrics and composer requirements """
-        if not self._composer.composer_requirements:
-            # Get all available operations for task
-            operations = get_operations_for_task(task=self.task, mode='all')
-
-            # Set protected attributes to composer
-            self._composer.composer_requirements = GPComposerRequirements(primary=operations, secondary=operations)
-        if not self._composer.metrics:
-            metric_function = ClassificationMetricsEnum.ROCAUC_penalty
-            if self.task.task_type in (TaskTypesEnum.regression, TaskTypesEnum.ts_forecasting):
-                metric_function = RegressionMetricsEnum.RMSE
-
-            # Set metric
-            self._composer.metrics = [metric_function]
-
-    def build(self) -> Composer:
-        optimiser_type = GPGraphOptimiser
-        if self.optimiser_parameters.genetic_scheme_type == GeneticSchemeTypesEnum.parameter_free:
-            optimiser_type = GPGraphParameterFreeOptimiser
-
-        graph_generation_params = GraphGenerationParams(adapter=PipelineAdapter(self._composer.log),
-                                                        advisor=PipelineChangeAdvisor())
-
-        archive_type = None
-        if len(self._composer.metrics) > 1:
-            archive_type = tools.ParetoFront()
-            # TODO add possibility of using regularization in MO alg
-            self.optimiser_parameters.regularization_type = RegularizationTypesEnum.none
-            self.optimiser_parameters.multi_objective = True
-
-        if self.optimiser_parameters.mutation_types is None:
-            self.optimiser_parameters.mutation_types = [boosting_mutation, parameter_change_mutation,
-                                                        MutationTypesEnum.single_edge,
-                                                        MutationTypesEnum.single_change,
-                                                        MutationTypesEnum.single_drop,
-                                                        MutationTypesEnum.single_add]
-
-        optimiser = optimiser_type(initial_graph=self._composer.initial_pipeline,
-                                   requirements=self._composer.composer_requirements,
-                                   graph_generation_params=graph_generation_params,
-                                   parameters=self.optimiser_parameters, log=self._composer.log,
-                                   archive_type=archive_type, metrics=self._composer.metrics)
-
-        self._composer.optimiser = optimiser
-
-        return self._composer
