@@ -25,7 +25,9 @@ def parameter_change_mutation(pipeline: Pipeline, requirements, **kwargs) -> Any
             # Perform specific change for particular parameter
             changer = ParametersChanger(operation_name, current_params)
             try:
-                node.custom_params = changer.get_new_operation_params()
+                new_params = changer.get_new_operation_params()
+                if new_params is not None:
+                    node.custom_params = new_params
             except Exception as ex:
                 pipeline.log.error(ex)
     return pipeline
