@@ -1,5 +1,5 @@
 from copy import copy
-from typing import List, TYPE_CHECKING
+from typing import TYPE_CHECKING, List
 
 if TYPE_CHECKING:
     from fedot.core.dag.graph_node import GraphNode
