@@ -1,6 +1,7 @@
 import glob
 import os
-from dataclasses import dataclass
+from copy import copy, deepcopy
+from dataclasses import dataclass, field
 from typing import List, Optional, Tuple, Union
 
 import imageio
@@ -10,14 +11,10 @@ from PIL import Image
 
 from fedot.core.data.load_data import JSONBatchLoader, TextBatchLoader
 from fedot.core.data.merge import DataMerger
-from fedot.core.data.multi_modal import MultiModalData
 from fedot.core.data.supplementary_data import SupplementaryData
 from fedot.core.repository.dataset_types import DataTypesEnum
 from fedot.core.repository.tasks import Task, TaskTypesEnum
 
-# Max unique values to convert numerical column to categorical.
-MAX_UNIQ_VAL = 12
-
 
 @dataclass
 class Data:
@@ -29,7 +26,7 @@ class Data:
     task: Task
     data_type: DataTypesEnum
     # Object with supplementary info
-    supplementary_data: SupplementaryData = SupplementaryData()
+    supplementary_data: SupplementaryData = field(default_factory=SupplementaryData)
 
     @staticmethod
     def from_csv(file_path=None,
@@ -55,7 +52,9 @@ class Data:
         # Get indices of the DataFrame
         data_array = np.array(data_frame).T
         idx = data_array[0]
-
+        if isinstance(idx[0], float) and idx[0] == round(idx[0]):
+            # if float indices is unnecessary
+            idx = [str(round(i)) for i in idx]
         if type(target_columns) is list:
             features, target = process_multiple_columns(target_columns, data_frame)
         else:
@@ -70,8 +69,15 @@ class Data:
                              delimiter=',',
                              is_predict=False,
                              target_column: Optional[str] = ''):
-        data_frame = pd.read_csv(file_path, sep=delimiter)
-        time_series = np.array(data_frame[target_column])
+        df = pd.read_csv(file_path, sep=delimiter)
+
+        idx = get_indices_from_file(df, file_path)
+
+        if target_column is not None:
+            time_series = np.array(df[target_column])
+        else:
+            time_series = np.array(df[df.columns[-1]])
+
         if is_predict:
             # Prepare data for prediction
             len_forecast = task.task_params.forecast_length
@@ -85,7 +91,7 @@ class Data:
                                    data_type=DataTypesEnum.ts)
         else:
             # Prepare InputData for train the pipeline
-            input_data = InputData(idx=np.arange(0, len(time_series)),
+            input_data = InputData(idx=idx,
                                    features=time_series,
                                    target=time_series,
                                    task=task,
@@ -175,7 +181,7 @@ class Data:
                         label: str = 'label',
                         task: Task = Task(TaskTypesEnum.classification),
                         data_type: DataTypesEnum = DataTypesEnum.table,
-                        export_to_meta=False) -> 'InputData':
+                        export_to_meta=False, is_multilabel=False) -> 'InputData':
         """
         Generates InputData from the set of JSON files with different fields
         :param files_path: path the folder with jsons
@@ -184,6 +190,7 @@ class Data:
         :param task: task to solve
         :param data_type: data type in fields (as well as type for obtained InputData)
         :param export_to_meta: combine extracted field and save to CSV
+        :param is_multilabel: if True, creates multilabel target
         :return: combined dataset
         """
 
@@ -202,10 +209,25 @@ class Data:
             val = df_data[fields_to_use[0]]
             # process field with nested list
             if isinstance(val[0], list):
-                val = [v[0] for v in val]
+                val = [' '.join(v) for v in val]
             features = np.array(val)
 
-        target = np.array(df_data[label])
+        if is_multilabel:
+            target = df_data[label]
+            classes = set()
+            for el in target:
+                for label in el:
+                    classes.add(label)
+            count_classes = list(sorted(classes))
+            multilabel_target = np.zeros((len(features), len(count_classes)))
+
+            for i in range(len(target)):
+                for el in target[i]:
+                    multilabel_target[i][count_classes.index(el)] = 1
+            target = multilabel_target
+        else:
+            target = np.array(df_data[label])
+
         idx = [index for index in range(len(target))]
 
         return InputData(idx=idx, features=features,
@@ -221,7 +243,7 @@ class InputData(Data):
 
     @property
     def num_classes(self) -> Optional[int]:
-        if self.task.task_type == TaskTypesEnum.classification:
+        if self.task.task_type == TaskTypesEnum.classification and self.target is not None:
             return len(np.unique(self.target))
         else:
             return None
@@ -232,17 +254,50 @@ class InputData(Data):
         # Update not only features but idx, target and task also
         idx, features, target, task, d_type, updated_info = DataMerger(outputs).merge()
 
+        # Mark data as preprocessed already
+        updated_info.was_preprocessed = True
         return InputData(idx=idx, features=features, target=target, task=task,
                          data_type=d_type, supplementary_data=updated_info)
 
-    def subset(self, start: int, end: int):
+    def subset_range(self, start: int, end: int):
         if not (0 <= start <= end <= len(self.idx)):
             raise ValueError('Incorrect boundaries for subset')
         new_features = None
         if self.features is not None:
             new_features = self.features[start:end + 1]
         return InputData(idx=self.idx[start:end + 1], features=new_features,
-                         target=self.target[start:end + 1], task=self.task, data_type=self.data_type)
+                         target=self.target[start:end + 1],
+                         task=self.task, data_type=self.data_type)
+
+    def subset_indices(self, selected_idx: List):
+        """
+        Get subset from InputData to extract all items with specified indices
+        :param selected_idx: list of indices for extraction
+        :return:
+        """
+        idx_list = [str(i) for i in self.idx]
+
+        # extractions of row number for each existing index from selected_idx
+        row_nums = [idx_list.index(str(selected_ind)) for selected_ind in selected_idx
+                    if str(selected_ind) in idx_list]
+        new_features = None
+
+        if self.features is not None:
+            new_features = self.features[row_nums]
+        return InputData(idx=np.asarray(self.idx)[row_nums], features=new_features,
+                         target=self.target[row_nums],
+                         task=self.task, data_type=self.data_type)
+
+    def subset_features(self, features_ids: list):
+        """ Return new InputData with subset of features based on features_ids list """
+        subsample_features = self.features[:, features_ids]
+        subsample_input = InputData(features=subsample_features,
+                                    data_type=self.data_type,
+                                    target=self.target, task=self.task,
+                                    idx=self.idx,
+                                    supplementary_data=self.supplementary_data)
+
+        return subsample_input
 
     def shuffle(self):
         """
@@ -250,13 +305,59 @@ class InputData(Data):
         """
         if self.data_type == DataTypesEnum.table:
             shuffled_ind = np.random.permutation(len(self.features))
-            idx, features, target = self.idx[shuffled_ind], self.features[shuffled_ind], self.target[shuffled_ind]
+            idx, features, target = np.asarray(self.idx)[shuffled_ind], self.features[shuffled_ind], self.target[
+                shuffled_ind]
             self.idx = idx
             self.features = features
             self.target = target
         else:
             pass
 
+    def convert_non_int_indexes_for_fit(self, pipeline):
+        """ Conversion non int (datetime, string, etc) indexes in integer form in fit stage """
+        copied_data = deepcopy(self)
+        is_timestamp = isinstance(copied_data.idx[0], pd._libs.tslibs.timestamps.Timestamp)
+        is_numpy_datetime = isinstance(copied_data.idx[0], np.datetime64)
+        # if fit stage- just creating range of integers
+        if is_timestamp or is_numpy_datetime:
+            copied_data.supplementary_data.non_int_idx = copy(copied_data.idx)
+            copied_data.idx = np.array(range(len(copied_data.idx)))
+
+            last_idx_time = copied_data.supplementary_data.non_int_idx[-1]
+            pre_last_time = copied_data.supplementary_data.non_int_idx[-2]
+
+            pipeline.last_idx_int = copied_data.idx[-1]
+            pipeline.last_idx_dt = last_idx_time
+            pipeline.period = last_idx_time - pre_last_time
+        elif type(copied_data.idx[0]) not in [int, np.int32, np.int64]:
+            copied_data.supplementary_data.non_int_idx = copy(copied_data.idx)
+            copied_data.idx = np.array(range(len(copied_data.idx)))
+            pipeline.last_idx_int = copied_data.idx[-1]
+        return copied_data
+
+    def convert_non_int_indexes_for_predict(self, pipeline):
+        """Conversion non int (datetime, string, etc) indexes in integer form in predict stage"""
+        copied_data = deepcopy(self)
+        is_timestamp = isinstance(copied_data.idx[0], pd._libs.tslibs.timestamps.Timestamp)
+        is_numpy_datetime = isinstance(copied_data.idx[0], np.datetime64)
+        # if predict stage - calculating shift from last train part index
+        if is_timestamp or is_numpy_datetime:
+            copied_data.supplementary_data.non_int_idx = copy(self.idx)
+            copied_data.idx = self._resolve_non_int_idx(pipeline)
+        elif type(copied_data.idx[0]) not in [int, np.int32, np.int64]:
+            # note, that string indexes do not have an order and always we think that indexes we want to predict go
+            # immediately after the train indexes
+            copied_data.supplementary_data.non_int_idx = copy(copied_data.idx)
+            copied_data.idx = pipeline.last_idx_int + np.array(range(1, len(copied_data.idx)+1))
+        return copied_data
+
+    @staticmethod
+    def _resolve_func(pipeline, x):
+        return pipeline.last_idx_int + (x - pipeline.last_idx_dt) // pipeline.period
+
+    def _resolve_non_int_idx(self, pipeline):
+        return np.array(list(map(lambda x: self._resolve_func(pipeline, x), self.idx)))
+
 
 @dataclass
 class OutputData(Data):
@@ -301,9 +402,10 @@ def process_one_column(target_column, data_frame, data_array):
         features = data_array[1:].T
         target = None
 
-    target = np.array(target)
-    if len(target.shape) < 2:
-        target = target.reshape((-1, 1))
+    if target is not None:
+        target = np.array(target)
+        if len(target.shape) < 2:
+            target = target.reshape((-1, 1))
 
     return features, target
 
@@ -318,105 +420,37 @@ def process_multiple_columns(target_columns, data_frame):
     return features, targets
 
 
-def data_has_categorical_features(data: Union[InputData, MultiModalData]) -> bool:
-    """ Check data for categorical columns. Also check, if some numerical column
-    has unique values less then MAX_UNIQ_VAL, then convert this column to string.
-
-    :param data: Union[InputData, MultiModalData]
-    :return data_has_categorical_columns: bool, whether data has categorical columns or not
-    """
-
-    data_has_categorical_columns = False
-
-    if isinstance(data, MultiModalData):
-        for data_source_name, values in data.items():
-            if data_source_name.startswith('data_source_table'):
-                data_has_categorical_columns = _has_data_categorical(values)
-    elif data_type_is_suitable_preprocessing(data):
-        data_has_categorical_columns = _has_data_categorical(data)
-
-    return data_has_categorical_columns
-
-
-def data_has_missing_values(data: Union[InputData, MultiModalData]) -> bool:
-    """ Check data for missing values."""
-
-    if isinstance(data, MultiModalData):
-        for data_source_name, values in data.items():
-            if data_type_is_table(values):
-                return pd.DataFrame(values.features).isna().sum().sum() > 0
-    elif data_type_is_suitable_preprocessing(data):
-        return pd.DataFrame(data.features).isna().sum().sum() > 0
-    return False
-
-
-def str_columns_check(features):
-    """
-    Method for checking which columns contain categorical (text) data
-
-    :param features: tabular data for check
-    :return categorical_ids: indices of categorical columns in table
-    :return non_categorical_ids: indices of non categorical columns in table
-    """
-    source_shape = features.shape
-    columns_amount = source_shape[1] if len(source_shape) > 1 else 1
-
-    categorical_ids = []
-    non_categorical_ids = []
-    # For every column in table make check for first element
-    for column_id in range(0, columns_amount):
-        column = features[:, column_id] if columns_amount > 1 else features
-        if isinstance(column[0], str):
-            categorical_ids.append(column_id)
-        else:
-            non_categorical_ids.append(column_id)
-
-    return categorical_ids, non_categorical_ids
-
-
-def divide_data_categorical_numerical(input_data: InputData) -> (InputData, InputData):
-    categorical_ids, non_categorical_ids = str_columns_check(input_data.features)
-    numerical_features = input_data.features[:, non_categorical_ids]
-    categorical_features = input_data.features[:, categorical_ids]
-
-    numerical = InputData(features=numerical_features, data_type=input_data.data_type,
-                          target=input_data.target, task=input_data.task, idx=input_data.idx)
-    categorical = InputData(features=categorical_features, data_type=input_data.data_type,
-                            target=input_data.target, task=input_data.task, idx=input_data.idx)
-
-    return numerical, categorical
-
-
-def data_type_is_table(data: InputData) -> bool:
+def data_type_is_table(data: Union[InputData, OutputData]) -> bool:
     return data.data_type == DataTypesEnum.table
 
 
-def data_type_is_suitable_preprocessing(data: InputData) -> bool:
-    if data.data_type == DataTypesEnum.table or data.data_type == DataTypesEnum.ts:
-        return True
-    return False
+def data_type_is_ts(data: InputData) -> bool:
+    return data.data_type == DataTypesEnum.ts
 
 
-def _has_data_categorical(data: InputData) -> bool:
-    """ Whether data categorical columns or not.
+def get_indices_from_file(data_frame, file_path):
+    if 'datetime' in data_frame.columns:
+        df = pd.read_csv(file_path,
+                         parse_dates=['datetime'])
+        idx = [str(d) for d in df['datetime']]
+        return idx
+    return np.arange(0, len(data_frame))
 
-    :param data: InputData
-    :return data_has_categorical_columns: bool, whether data has categorical columns or not
-    """
-    data_has_categorical_columns = False
 
-    if isinstance(data.features, list) or len(data.features.shape) == 1:
-        data_has_categorical_columns = _is_values_categorical(data.features)
+def array_to_input_data(features_array: np.array,
+                        target_array: np.array,
+                        idx: Optional[np.array] = None,
+                        task: Task = Task(TaskTypesEnum.classification)):
+    data_type = autodetect_data_type(task)
+
+    if idx is None:
+        idx = np.arange(len(features_array))
+
+    return InputData(idx=idx, features=features_array, target=target_array, task=task, data_type=data_type)
+
+
+def autodetect_data_type(task: Task) -> DataTypesEnum:
+    if task.task_type == TaskTypesEnum.ts_forecasting:
+        return DataTypesEnum.ts
     else:
-        num_columns = data.features.shape[1]
-        for col_index in range(num_columns):
-            if data_has_categorical_columns:
-                break
-            data_has_categorical_columns = _is_values_categorical(data.features[:, col_index])
-
-    return data_has_categorical_columns
-
-
-def _is_values_categorical(values: List):
-    # Check if any value in list has 'string' type
-    return any(list(map(lambda x: isinstance(x, str), values)))
+        return DataTypesEnum.table
