@@ -1,7 +1,7 @@
+from copy import deepcopy
 from typing import Tuple, Union
 
-import numpy as np
-from sklearn.model_selection import KFold, train_test_split
+from sklearn.model_selection import train_test_split
 
 from fedot.core.data.data import InputData
 from fedot.core.data.multi_modal import MultiModalData
@@ -26,25 +26,17 @@ def _split_time_series(data, task, *args, **kwargs):
     y_train = input_target[:-forecast_length]
     y_test = input_target[-forecast_length:]
 
-    idx_for_train = np.arange(0, len(x_train))
-
-    # Define indices for test
-    start_forecast = len(x_train)
-    end_forecast = start_forecast + forecast_length
-    idx_for_predict = np.arange(start_forecast, end_forecast)
+    idx_train = data.idx[:-forecast_length]
+    idx_test = data.idx[-forecast_length:]
 
     # Prepare data to train the operation
-    train_data = InputData(idx=idx_for_train,
-                           features=x_train,
-                           target=y_train,
-                           task=task,
-                           data_type=DataTypesEnum.ts)
+    train_data = InputData(idx=idx_train, features=x_train, target=y_train,
+                           task=task, data_type=DataTypesEnum.ts,
+                           supplementary_data=data.supplementary_data)
 
-    test_data = InputData(idx=idx_for_predict,
-                          features=x_test,
-                          target=y_test,
-                          task=task,
-                          data_type=DataTypesEnum.ts)
+    test_data = InputData(idx=idx_test, features=x_test, target=y_test,
+                          task=task, data_type=DataTypesEnum.ts,
+                          supplementary_data=data.supplementary_data)
 
     return train_data, test_data
 
@@ -66,28 +58,23 @@ def _split_any(data, task, data_type, split_ratio, with_shuffle=False):
     # Predictors and target
     input_features = data.features
     input_target = data.target
-
-    x_train, x_test, y_train, y_test = train_test_split(input_features,
-                                                        input_target,
-                                                        test_size=1. - split_ratio,
-                                                        shuffle=with_shuffle,
-                                                        random_state=random_state)
-
-    idx_for_train = np.arange(0, len(x_train))
-    idx_for_predict = np.arange(0, len(x_test))
+    idx = data.idx
+    idx_train, idx_test, x_train, x_test, y_train, y_test = \
+        train_test_split(idx,
+                         input_features,
+                         input_target,
+                         test_size=1. - split_ratio,
+                         shuffle=with_shuffle,
+                         random_state=random_state)
 
     # Prepare data to train the operation
-    train_data = InputData(idx=idx_for_train,
-                           features=x_train,
-                           target=y_train,
-                           task=task,
-                           data_type=data_type)
+    train_data = InputData(idx=idx_train, features=x_train,  target=y_train,
+                           task=task, data_type=data_type,
+                           supplementary_data=data.supplementary_data)
 
-    test_data = InputData(idx=idx_for_predict,
-                          features=x_test,
-                          target=y_test,
-                          task=task,
-                          data_type=data_type)
+    test_data = InputData(idx=idx_test, features=x_test, target=y_test,
+                          task=task, data_type=data_type,
+                          supplementary_data=data.supplementary_data)
 
     return train_data, test_data
 
@@ -124,7 +111,7 @@ def _split_text(data, task, split_ratio, with_shuffle=False):
     :param with_shuffle: is data needed to be shuffled or not
     """
 
-    return _split_any(data, task, DataTypesEnum.image, split_ratio, with_shuffle)
+    return _split_any(data, task, DataTypesEnum.text, split_ratio, with_shuffle)
 
 
 def _train_test_single_data_setup(data: InputData, split_ratio=0.8,
@@ -156,6 +143,9 @@ def _train_test_single_data_setup(data: InputData, split_ratio=0.8,
     else:
         raise ValueError('InputData must be not empty')
 
+    # Store additional information
+    train_data.supplementary_data = deepcopy(data.supplementary_data)
+    test_data.supplementary_data = deepcopy(data.supplementary_data)
     return train_data, test_data
 
 
