@@ -1,12 +1,14 @@
 from typing import Optional
 
 import numpy as np
+import pandas as pd
 from sklearn.decomposition import KernelPCA, PCA
 from sklearn.impute import SimpleImputer
-from sklearn.preprocessing import MinMaxScaler, OneHotEncoder, PolynomialFeatures, StandardScaler
+from sklearn.preprocessing import MinMaxScaler, PolynomialFeatures, StandardScaler
 
-from fedot.core.data.data import InputData
-from fedot.core.data.data import data_has_categorical_features, divide_data_categorical_numerical, str_columns_check
+from fedot.core.data.data import InputData, data_type_is_table, OutputData
+from fedot.core.data.data_preprocessing import replace_inf_with_nans, convert_into_column, \
+    divide_data_categorical_numerical, find_categorical_columns, data_has_categorical_features
 from fedot.core.operations.evaluation.operation_implementations. \
     implementation_interfaces import DataOperationImplementation, EncodedInvariantImplementation
 
@@ -22,6 +24,8 @@ class ComponentAnalysisImplementation(DataOperationImplementation):
         self.pca = None
         self.params = None
         self.number_of_features = None
+        self.number_of_samples = None
+
         self.parameters_changed = False
 
     def fit(self, input_data):
@@ -31,10 +35,10 @@ class ComponentAnalysisImplementation(DataOperationImplementation):
         :param input_data: data with features, target and ids for PCA training
         :return pca: trained PCA model (optional output)
         """
-        self.number_of_features = np.array(input_data.features).shape[1]
+        self.number_of_samples, self.number_of_features = np.array(input_data.features).shape
 
         if self.number_of_features > 1:
-            self.parameters_changed = self.check_and_correct_params()
+            self.check_and_correct_params()
             self.pca.fit(input_data.features)
 
         return self.pca
@@ -56,25 +60,28 @@ class ComponentAnalysisImplementation(DataOperationImplementation):
         # Update features
         output_data = self._convert_to_output(input_data,
                                               transformed_features)
+        self.update_column_types(output_data)
         return output_data
 
-    def check_and_correct_params(self) -> bool:
-        """ Method check if amount of features in data enough for n_components
+    def check_and_correct_params(self):
+        """ Method check if number of features in data enough for n_components
         parameter in PCA or not. And if not enough - fixes it
         """
-        was_changed = False
         current_parameters = self.pca.get_params()
 
         if type(current_parameters['n_components']) == int:
             if current_parameters['n_components'] > self.number_of_features:
                 current_parameters['n_components'] = self.number_of_features
-                was_changed = True
+                self.parameters_changed = True
+        elif current_parameters['n_components'] == 'mle':
+            # Check that n_samples correctly map with n_features
+            if self.number_of_samples < self.number_of_features:
+                current_parameters['n_components'] = 0.5
+                self.parameters_changed = True
 
         self.pca.set_params(**current_parameters)
         self.params = current_parameters
 
-        return was_changed
-
     def get_params(self):
         if self.parameters_changed is True:
             params_dict = self.pca.get_params()
@@ -82,6 +89,13 @@ class ComponentAnalysisImplementation(DataOperationImplementation):
         else:
             return self.pca.get_params()
 
+    @staticmethod
+    def update_column_types(output_data: OutputData):
+        """ Update column types after applying PCA operations """
+        n_rows, n_cols = output_data.predict.shape
+        output_data.supplementary_data.column_types['features'] = [str(float) * n_cols]
+        return output_data
+
 
 class PCAImplementation(ComponentAnalysisImplementation):
     """ Class for applying PCA from sklearn
@@ -97,7 +111,7 @@ class PCAImplementation(ComponentAnalysisImplementation):
         else:
             self.pca = PCA(**params)
         self.params = params
-        self.amount_of_features = None
+        self.number_of_features = None
 
 
 class KernelPCAImplementation(ComponentAnalysisImplementation):
@@ -116,98 +130,6 @@ class KernelPCAImplementation(ComponentAnalysisImplementation):
         self.params = params
 
 
-class OneHotEncodingImplementation(DataOperationImplementation):
-    """ Class for automatic categorical data detection and one hot encoding """
-
-    def __init__(self, **params: Optional[dict]):
-        super().__init__()
-        default_params = {
-            'drop': 'if_binary'
-        }
-        if not params:
-            # Default parameters
-            self.encoder = OneHotEncoder(**default_params)
-        else:
-            self.encoder = OneHotEncoder(**{**params, **default_params})
-        self.categorical_ids = None
-        self.non_categorical_ids = None
-
-    def fit(self, input_data: InputData):
-        """ Method for fit encoder with automatic determination of categorical features
-
-        :param input_data: data with features, target and ids for encoder training
-        :return encoder: trained encoder (optional output)
-        """
-        features = input_data.features
-        categorical_ids, non_categorical_ids = str_columns_check(features)
-
-        # Indices of columns with categorical and non-categorical features
-        self.categorical_ids = categorical_ids
-        self.non_categorical_ids = non_categorical_ids
-
-        if len(categorical_ids) == 0:
-            pass
-        else:
-            categorical_features = np.array(features[:, categorical_ids])
-            self.encoder.fit(categorical_features)
-
-    def transform(self, input_data, is_fit_pipeline_stage: Optional[bool]):
-        """
-        The method that transforms the categorical features in the original
-        dataset, but does not affect the rest features
-
-        :param input_data: data with features, target and ids for transformation
-        :param is_fit_pipeline_stage: is this fit or predict stage for pipeline
-        :return output_data: output data with transformed features table
-        """
-
-        features = input_data.features
-        if len(self.categorical_ids) == 0:
-            # If there are no categorical features in the table
-            transformed_features = features
-        else:
-            # If categorical features are exists
-            transformed_features = self._make_new_table(features)
-
-        # Update features
-        output_data = self._convert_to_output(input_data,
-                                              transformed_features)
-        return output_data
-
-    def _make_new_table(self, features):
-        """
-        The method creates a table based on categorical and real features
-
-        :param features: tabular data for processing
-        :return transformed_features: transformed features table
-        """
-
-        categorical_features = np.array(features[:, self.categorical_ids])
-        self._check_same_categories(categorical_features)
-        transformed_categorical = self.encoder.transform(categorical_features).toarray()
-
-        # If there are non-categorical features in the data
-        if len(self.non_categorical_ids) == 0:
-            transformed_features = transformed_categorical
-        else:
-            # Stack transformed categorical and non-categorical data
-            non_categorical_features = np.array(features[:, self.non_categorical_ids])
-            frames = (non_categorical_features, transformed_categorical)
-            transformed_features = np.hstack(frames)
-
-        return transformed_features
-
-    def _check_same_categories(self, categorical_features):
-        encoder_unique_categories = sorted(list(np.hstack(self.encoder.categories_)))
-        features_unique_categories = sorted(np.unique(np.array(categorical_features)))
-
-        if encoder_unique_categories != features_unique_categories:
-            raise ValueError('Category in test data was not exist in train.')
-
-    def get_params(self):
-        return self.encoder.get_params()
-
-
 class PolyFeaturesImplementation(EncodedInvariantImplementation):
     """ Class for application of PolynomialFeatures operation on data,
     where only not encoded features (were not converted from categorical using
@@ -232,6 +154,18 @@ class PolyFeaturesImplementation(EncodedInvariantImplementation):
     def get_params(self):
         return self.operation.get_params()
 
+    def _update_column_types(self, source_features_shape, output_data: OutputData):
+        """ Update column types after applying operations. If new columns added, new type for them are defined """
+        if len(source_features_shape) < 2:
+            return output_data
+        else:
+            cols_number_added = output_data.predict.shape[1] - source_features_shape[1]
+            if cols_number_added > 0:
+                # There are new columns in the table
+                col_types = output_data.supplementary_data.column_types['features']
+                col_types.extend([str(float)] * cols_number_added)
+                output_data.supplementary_data.column_types['features'] = col_types
+
 
 class ScalingImplementation(EncodedInvariantImplementation):
     """ Class for application of Scaling operation on data,
@@ -286,6 +220,9 @@ class ImputationImplementation(DataOperationImplementation):
         default_params_categorical = {'strategy': 'most_frequent'}
         self.params_cat = {**params, **default_params_categorical}
         self.params_num = params
+        self.categorical_ids = None
+        self.non_categorical_ids = None
+        self.ids_binary_integer_features = {}
 
         if not params:
             # Default parameters
@@ -300,30 +237,29 @@ class ImputationImplementation(DataOperationImplementation):
         The method trains SimpleImputer
 
         :param input_data: data with features
-        :return imputer: trained SimpleImputer model
         """
 
-        features_with_replaced_inf = np.where(np.isin(input_data.features,
-                                                      [np.inf, -np.inf]),
-                                              np.nan,
-                                              input_data.features)
-        input_data.features = features_with_replaced_inf
+        replace_inf_with_nans(input_data)
 
-        if data_has_categorical_features(input_data):
-            numerical, categorical = divide_data_categorical_numerical(input_data)
-            if len(categorical.features.shape) == 1:
-                self.imputer_cat.fit(categorical.features.reshape(-1, 1))
-            else:
+        if data_type_is_table(input_data):
+            # Tabular data contains categorical features
+            categorical_ids, non_categorical_ids = find_categorical_columns(input_data.features)
+            numerical, categorical = divide_data_categorical_numerical(input_data, categorical_ids,
+                                                                       non_categorical_ids)
+
+            if categorical is not None and categorical.features.size > 0:
+                categorical.features = convert_into_column(categorical.features)
+                # Imputing for categorical values
                 self.imputer_cat.fit(categorical.features)
-            if len(numerical.features.shape) == 1:
-                self.imputer_num.fit(numerical.features.reshape(-1, 1))
-            else:
+
+            if numerical is not None and numerical.features.size > 0:
+                numerical.features = convert_into_column(numerical.features)
+                # Imputing for numerical values
                 self.imputer_num.fit(numerical.features)
         else:
-            if len(input_data.features.shape) == 1:
-                self.imputer_num.fit(input_data.features.reshape(-1, 1))
-            else:
-                self.imputer_num.fit(input_data.features)
+            # Time series or other type of non-tabular data
+            input_data.features = convert_into_column(input_data.features)
+            self.imputer_num.fit(input_data.features)
 
     def transform(self, input_data, is_fit_pipeline_stage: Optional[bool] = None):
         """
@@ -333,28 +269,41 @@ class ImputationImplementation(DataOperationImplementation):
         :param is_fit_pipeline_stage: is this fit or predict stage for pipeline
         :return input_data: data with transformed features attribute
         """
-        features_with_replaced_inf = np.where(np.isin(input_data.features,
-                                                      [np.inf, -np.inf]),
-                                              np.nan,
-                                              input_data.features)
-        input_data.features = features_with_replaced_inf
+        replace_inf_with_nans(input_data)
+
+        if data_type_is_table(input_data) and data_has_categorical_features(input_data):
+            features_types = input_data.supplementary_data.column_types.get('features')
+            self.categorical_ids, self.non_categorical_ids = find_categorical_columns(input_data.features,
+                                                                                      features_types)
+            numerical, categorical = divide_data_categorical_numerical(input_data, self.categorical_ids,
+                                                                       self.non_categorical_ids)
+
+            if categorical is not None:
+                categorical_features = convert_into_column(categorical.features)
+                categorical_features = self.imputer_cat.transform(categorical_features)
+
+            if numerical is not None:
+                numerical_features = convert_into_column(numerical.features)
+
+                # Features with only two unique values must be filled in a specific way
+                self._find_binary_features(numerical_features)
+                numerical_features = self.imputer_num.transform(numerical_features)
+                numerical_features = self._correct_binary_ids_features(numerical_features)
+
+            if categorical is not None and numerical is not None:
+                # Stack both categorical and numerical features
+                transformed_features = self._categorical_numerical_union(categorical_features,
+                                                                         numerical_features)
+            elif categorical is not None and numerical is None:
+                # Dataset contain only categorical features
+                transformed_features = categorical_features
+            elif categorical is None and numerical is not None:
+                # Dataset contain only numerical features
+                transformed_features = numerical_features
 
-        if data_has_categorical_features(input_data):
-            numerical, categorical = divide_data_categorical_numerical(input_data)
-            if len(categorical.features.shape) == 1:
-                categorical_features = self.imputer_cat.transform(categorical.features.reshape(-1, 1))
-            else:
-                categorical_features = self.imputer_cat.transform(categorical.features)
-            if len(numerical.features.shape) == 1:
-                numerical_features = self.imputer_num.transform(numerical.features.reshape(-1, 1))
-            else:
-                numerical_features = self.imputer_num.transform(numerical.features)
-            transformed_features = np.hstack((categorical_features, numerical_features))
         else:
-            if len(input_data.features.shape) == 1:
-                transformed_features = self.imputer_num.transform(input_data.features.reshape(-1, 1))
-            else:
-                transformed_features = self.imputer_num.transform(input_data.features)
+            input_data.features = convert_into_column(input_data.features)
+            transformed_features = self.imputer_num.transform(input_data.features)
 
         output_data = self._convert_to_output(input_data, transformed_features, data_type=input_data.data_type)
         return output_data
@@ -371,6 +320,54 @@ class ImputationImplementation(DataOperationImplementation):
         output_data = self.transform(input_data)
         return output_data
 
+    def _categorical_numerical_union(self, categorical_features: np.array, numerical_features: np.array):
+        """ Merge numerical and categorical features in right order (as it was in source table) """
+        categorical_df = pd.DataFrame(categorical_features, columns=self.categorical_ids)
+        numerical_df = pd.DataFrame(numerical_features, columns=self.non_categorical_ids)
+        all_features_df = pd.concat([numerical_df, categorical_df], axis=1)
+
+        # Sort column names
+        all_features_df = all_features_df.sort_index(axis=1)
+        return np.array(all_features_df)
+
+    def _find_binary_features(self, numerical_features: np.array):
+        """ Find indices of features with only two unique values in column.
+        All features in table are numerical.
+        """
+        df = pd.DataFrame(numerical_features)
+
+        # Calculate unique values per column (excluding nans)
+        for column_id, col in enumerate(df):
+            unique_values = df[col].dropna().unique()
+            if len(unique_values) <= 2:
+                # Current numerical column has only two values
+                column_info = {column_id: {'min': min(unique_values),
+                                           'max': max(unique_values)}}
+                self.ids_binary_integer_features.update(column_info)
+
+    def _correct_binary_ids_features(self, filled_numerical_features: np.array) -> np.array:
+        """ Correct filled features if previously it was binary. Discretization is performed
+        for the reconstructed values.
+        For example, [1, 1, 0.75, 0] will be transformed into [1, 1, 1, 0]
+        """
+        list_binary_ids = list(self.ids_binary_integer_features.keys())
+        if len(list_binary_ids) == 0:
+            # Return source array
+            return filled_numerical_features
+
+        for bin_id in list_binary_ids:
+            # Correct values inplace
+            filled_column = filled_numerical_features[:, bin_id]
+            min_value = self.ids_binary_integer_features[bin_id]['min']
+            max_value = self.ids_binary_integer_features[bin_id]['max']
+            mean_value = (max_value - min_value) / 2
+
+            filled_column[filled_column > mean_value] = max_value
+            filled_column[filled_column < mean_value] = min_value
+
+        return filled_numerical_features
+
     def get_params(self) -> dict:
-        dictionary = {'imputer_categorical': self.params_cat, 'imputer_numerical': self.params_num}
-        return dictionary
+        features_imputers = {'imputer_categorical': self.params_cat,
+                             'imputer_numerical': self.params_num}
+        return features_imputers
