@@ -42,7 +42,8 @@ class TextCleanImplementation(DataOperationImplementation):
 
         clean_data = []
         for text in input_data.features:
-            words = set(self._word_vectorize(text))
+            text = str(text).lower()
+            words = self._word_vectorize(text)
             without_stop_words = self._remove_stop_words(words)
             words = self._lemmatization(without_stop_words)
             words = [word for word in words if word.isalpha()]
@@ -58,7 +59,7 @@ class TextCleanImplementation(DataOperationImplementation):
 
     @staticmethod
     def _download_nltk_resources():
-        for resource in ['punkt', 'stopwords', 'wordnet']:
+        for resource in ['punkt', 'stopwords', 'wordnet', 'omw-1.4']:
             try:
                 nltk.data.find(f'tokenizers/{resource}')
             except LookupError:
@@ -98,4 +99,4 @@ class TextCleanImplementation(DataOperationImplementation):
         return text
 
     def get_params(self):
-        return {'language': self.lang}
+        raise NotImplementedError()
