@@ -5,15 +5,21 @@ from fedot.core.data.data import InputData, OutputData
 from fedot.core.operations.evaluation.evaluation_interfaces import EvaluationStrategy
 from fedot.core.operations.evaluation.operation_implementations.data_operations.ts_transformations import \
     ExogDataTransformationImplementation, GaussianFilterImplementation, LaggedTransformationImplementation, \
-    TsSmoothingImplementation, SparseLaggedTransformationImplementation
+    TsSmoothingImplementation, SparseLaggedTransformationImplementation, CutImplementation, \
+    NumericalDerivativeFilterImplementation
 from fedot.core.operations.evaluation.operation_implementations.models. \
-    ts_implementations import ARIMAImplementation, AutoRegImplementation, STLForecastARIMAImplementation, \
+    ts_implementations.statsmodels import AutoRegImplementation, GLMImplementation, ExpSmoothingImplementation
+from fedot.core.operations.evaluation.operation_implementations.models.ts_implementations.arima import \
+    ARIMAImplementation, STLForecastARIMAImplementation
+from fedot.core.operations.evaluation.operation_implementations.models.ts_implementations.clstm import \
     CLSTMImplementation
+from fedot.core.operations.evaluation.operation_implementations.models.ts_implementations.poly import \
+    PolyfitImplementation
 
 warnings.filterwarnings("ignore", category=UserWarning)
 
 
-class CustomTsForecastingStrategy(EvaluationStrategy):
+class FedotTsForecastingStrategy(EvaluationStrategy):
     """
     This class defines the certain classical models implementation for time
     series forecasting (e.g. AR, ARIMA)
@@ -27,7 +33,11 @@ class CustomTsForecastingStrategy(EvaluationStrategy):
         'arima': ARIMAImplementation,
         'ar': AutoRegImplementation,
         'stl_arima': STLForecastARIMAImplementation,
-        'clstm': CLSTMImplementation}
+        'ets': ExpSmoothingImplementation,
+        'clstm': CLSTMImplementation,
+        'polyfit': PolyfitImplementation,
+        'glm': GLMImplementation
+    }
 
     def __init__(self, operation_type: str, params: Optional[dict] = None):
         super().__init__(operation_type, params)
@@ -74,7 +84,7 @@ class CustomTsForecastingStrategy(EvaluationStrategy):
                              f'strategy for {operation_type}')
 
 
-class CustomTsTransformingStrategy(EvaluationStrategy):
+class FedotTsTransformingStrategy(EvaluationStrategy):
     """
     This class defines the certain data operation implementation for time series
     forecasting
@@ -89,7 +99,9 @@ class CustomTsTransformingStrategy(EvaluationStrategy):
         'sparse_lagged': SparseLaggedTransformationImplementation,
         'smoothing': TsSmoothingImplementation,
         'exog_ts': ExogDataTransformationImplementation,
-        'gaussian_filter': GaussianFilterImplementation}
+        'gaussian_filter': GaussianFilterImplementation,
+        'diff_filter': NumericalDerivativeFilterImplementation,
+        'cut': CutImplementation}
 
     def __init__(self, operation_type: str, params: Optional[dict] = None):
         super().__init__(operation_type, params)
