@@ -4,7 +4,6 @@ from io import BytesIO
 
 import joblib
 import numpy as np
-
 from fedot.core.log import Log, default_log
 from fedot.core.pipelines.node import Node
 
@@ -118,8 +117,11 @@ class OperationTemplate(OperationTemplateAbstract):
         return params
 
     def _extract_fields_of_fitted_operation(self, node: Node):
-        operation_name = f'operation_{str(self.operation_id)}.pkl'
-        self.fitted_operation_path = os.path.join('fitted_operations', operation_name)
+        if 'h2o' in self.operation_type:
+            self.fitted_operation_path = os.path.join('fitted_operations', f"h2o_{self.operation_id}")
+        else:
+            operation_name = f'operation_{str(self.operation_id)}.pkl'
+            self.fitted_operation_path = os.path.join('fitted_operations', operation_name)
         self.fitted_operation = node.fitted_operation
 
     def convert_to_dict(self) -> dict:
@@ -139,14 +141,22 @@ class OperationTemplate(OperationTemplateAbstract):
 
     def export_operation(self, path: str = None):
         if path:
-            _check_existing_path(path)
+            check_existing_path(path)
 
             # dictionary with paths to saved fitted operations
             if self.fitted_operation:
-                path_fitted_operations = os.path.join(path, 'fitted_operations')
-                _check_existing_path(path_fitted_operations)
-                joblib.dump(self.fitted_operation, os.path.join(path, self.fitted_operation_path))
-                return os.path.join(path, 'fitted_operations', f'operation_{self.operation_id}.pkl')
+                if 'h2o' in self.operation_type:
+                    self.fitted_operation_path = self.fitted_operation.save_operation(
+                        os.path.join(path, 'fitted_operations'),
+                        self.operation_id
+                    )
+
+                    return self.fitted_operation_path
+                else:
+                    path_fitted_operations = os.path.join(path, 'fitted_operations')
+                    check_existing_path(path_fitted_operations)
+                    joblib.dump(self.fitted_operation, os.path.join(path, self.fitted_operation_path))
+                    return os.path.join(path_fitted_operations, f'operation_{self.operation_id}.pkl')
             else:
                 return None
         else:
@@ -178,7 +188,7 @@ class OperationTemplate(OperationTemplateAbstract):
             self.rating = operation_object['rating']
 
 
-def _check_existing_path(path: str):
+def check_existing_path(path: str):
     if not os.path.exists(path):
         os.makedirs(path)
 
