@@ -1,12 +1,15 @@
+import timeit
 import warnings
 from copy import deepcopy
 from random import choice, randint
-from typing import (Any, Callable, List, Tuple)
+from typing import Any, Callable, List, Tuple, Union
 
 from fedot.core.composer.constraint import constraint_function
+from fedot.core.log import default_log
 from fedot.core.optimisers.graph import OptGraph, OptNode
 from fedot.core.optimisers.utils.multi_objective_fitness import MultiObjFitness
 from fedot.core.utils import DEFAULT_PARAMS_STUB
+from fedot.remote.remote_evaluator import RemoteEvaluator
 
 MAX_ITERS = 1000
 
@@ -29,7 +32,7 @@ def random_graph(params, requirements, max_depth=None) -> OptGraph:
         n_iter += 1
         if n_iter > MAX_ITERS:
             warnings.warn(f'Random_graph generation failed for {n_iter} iterations.')
-            raise ValueError(f'Random_graph generation failed. Cannot find valid pipeline'
+            raise ValueError(f'Random_graph generation failed. Cannot find valid graph'
                              f' with current params {params} and requirements {requirements}')
 
     return graph
@@ -111,12 +114,30 @@ def num_of_parents_in_crossover(num_of_final_inds: int) -> int:
 
 def evaluate_individuals(individuals_set, objective_function, graph_generation_params,
                          is_multi_objective: bool, timer=None):
+    logger = default_log('individuals evaluation logger')
+
     num_of_successful_evals = 0
     reversed_set = individuals_set[::-1]
+
+    # TODO refactor
+    fitter = RemoteEvaluator()
+    pre_evaluated_objects = []
+    if fitter.use_remote:
+        logger.info('Remote fit used')
+        restored_graphs = [graph_generation_params.adapter.restore(ind.graph) for ind in reversed_set]
+        pre_evaluated_objects = fitter.compute_pipelines(restored_graphs)
+
     evaluated_individuals = []
-    for ind in reversed_set:
-        ind.fitness = calculate_objective(ind.graph, objective_function,
+
+    for ind_num, ind in enumerate(reversed_set):
+        start_time = timeit.default_timer()
+
+        graph = ind.graph
+        if len(pre_evaluated_objects) > 0:
+            graph = pre_evaluated_objects[ind_num]
+        ind.fitness = calculate_objective(graph, objective_function,
                                           is_multi_objective, graph_generation_params)
+        ind.computation_time = timeit.default_timer() - start_time
         if ind.fitness is not None:
             num_of_successful_evals += 1
             evaluated_individuals.append(ind)
@@ -128,11 +149,14 @@ def evaluate_individuals(individuals_set, objective_function, graph_generation_p
     return evaluated_individuals
 
 
-def calculate_objective(graph: OptGraph, objective_function: Callable, is_multi_objective: bool,
+def calculate_objective(graph: Union[OptGraph, Any], objective_function: Callable,
+                        is_multi_objective: bool,
                         graph_generation_params) -> Any:
-    # Transform OptGraph into Pipeline
-    pipeline = graph_generation_params.adapter.restore(graph)
-    calculated_fitness = objective_function(pipeline)
+    if isinstance(graph, OptGraph):
+        converted_object = graph_generation_params.adapter.restore(graph)
+    else:
+        converted_object = graph
+    calculated_fitness = objective_function(converted_object)
     if calculated_fitness is None:
         return None
     else:
@@ -163,4 +187,4 @@ def duplicates_filtration(archive, population):
 
 def clean_operators_history(population):
     for ind in population:
-        ind.parent_operator = []
+        ind.parent_operators = []
