@@ -5,22 +5,27 @@ from functools import partial
 from typing import (Any, Callable, List, Optional, Tuple, Union)
 
 import numpy as np
+from tqdm import tqdm
+from deap.tools import ParetoFront
 
-from fedot.core.composer.advisor import DefaultChangeAdvisor
 from fedot.core.composer.constraint import constraint_function
-from fedot.core.log import Log, default_log
-from fedot.core.optimisers.adapters import BaseOptimizationAdapter, DirectAdapter
+from fedot.core.log import Log
 from fedot.core.optimisers.gp_comp.archive import SimpleArchive
-from fedot.core.optimisers.gp_comp.gp_operators import clean_operators_history, \
-    duplicates_filtration, evaluate_individuals, num_of_parents_in_crossover, random_graph
+from fedot.core.optimisers.gp_comp.gp_operators import (
+    clean_operators_history,
+    duplicates_filtration,
+    evaluate_individuals,
+    num_of_parents_in_crossover,
+    random_graph
+)
 from fedot.core.optimisers.gp_comp.individual import Individual
 from fedot.core.optimisers.gp_comp.operators.crossover import CrossoverTypesEnum, crossover
 from fedot.core.optimisers.gp_comp.operators.inheritance import GeneticSchemeTypesEnum, inheritance
 from fedot.core.optimisers.gp_comp.operators.mutation import MutationTypesEnum, mutation
-from fedot.core.optimisers.gp_comp.operators.regularization import RegularizationTypesEnum, \
-    regularized_population
+from fedot.core.optimisers.gp_comp.operators.regularization import RegularizationTypesEnum, regularized_population
 from fedot.core.optimisers.gp_comp.operators.selection import SelectionTypesEnum, selection
-from fedot.core.optimisers.opt_history import OptHistory
+from fedot.core.optimisers.graph import OptGraph
+from fedot.core.optimisers.optimizer import GraphOptimiser, GraphOptimiserParameters, correct_if_has_nans
 from fedot.core.optimisers.timer import OptimisationTimer
 from fedot.core.optimisers.utils.population_utils import is_equal_archive, is_equal_fitness
 from fedot.core.repository.quality_metrics_repository import MetricsEnum
@@ -29,7 +34,7 @@ MAX_NUM_OF_GENERATED_INDS = 10000
 MIN_POPULATION_SIZE_WITH_ELITISM = 2
 
 
-class GPGraphOptimiserParameters:
+class GPGraphOptimiserParameters(GraphOptimiserParameters):
     """
         This class is for defining the parameters of optimiser
 
@@ -42,28 +47,9 @@ class GPGraphOptimiserParameters:
         evolution. Default False.
         :param depth_increase_step: the step of depth increase in automated depth configuration
         :param multi_objective: flag used for of algorithm type definition (muti-objective if true or  single-objective
-        if false). Value is defined in GPComposerBuilder. Default False.
+        if false). Value is defined in ComposerBuilder. Default False.
     """
 
-    def __init__(self, selection_types: List[SelectionTypesEnum] = None,
-                 crossover_types: List[Union[CrossoverTypesEnum, Any]] = None,
-                 mutation_types: List[Union[MutationTypesEnum, Any]] = None,
-                 regularization_type: RegularizationTypesEnum = RegularizationTypesEnum.none,
-                 genetic_scheme_type: GeneticSchemeTypesEnum = GeneticSchemeTypesEnum.generational,
-                 with_auto_depth_configuration: bool = False, depth_increase_step: int = 3,
-                 multi_objective: bool = False,
-                 history_folder: str = None):
-
-        self.selection_types = selection_types
-        self.crossover_types = crossover_types
-        self.mutation_types = mutation_types
-        self.regularization_type = regularization_type
-        self.genetic_scheme_type = genetic_scheme_type
-        self.with_auto_depth_configuration = with_auto_depth_configuration
-        self.depth_increase_step = depth_increase_step
-        self.multi_objective = multi_objective
-        self.history_folder = history_folder
-
     def set_default_params(self):
         """
         Choose default configuration of the evolutionary operators
@@ -84,37 +70,46 @@ class GPGraphOptimiserParameters:
                                    MutationTypesEnum.growth,
                                    MutationTypesEnum.local_growth]
 
+    def __init__(self, selection_types: List[SelectionTypesEnum] = None,
+                 crossover_types: List[Union[CrossoverTypesEnum, Any]] = None,
+                 mutation_types: List[Union[MutationTypesEnum, Any]] = None,
+                 regularization_type: RegularizationTypesEnum = RegularizationTypesEnum.none,
+                 genetic_scheme_type: GeneticSchemeTypesEnum = GeneticSchemeTypesEnum.generational,
+                 archive_type=None,
+                 *args, **kwargs):
 
-class GPGraphOptimiser:
+        super().__init__(*args, **kwargs)
+
+        self.selection_types = selection_types
+        self.crossover_types = crossover_types
+        self.mutation_types = mutation_types
+        self.regularization_type = regularization_type
+        self.genetic_scheme_type = genetic_scheme_type
+        self.archive_type = archive_type
+
+
+class EvoGraphOptimiser(GraphOptimiser):
     """
-    Base class of evolutionary graph optimiser
-
-    :param initial_graph: graph which was initialized outside the optimiser
-    :param requirements: optimizer requirements
-    :param graph_generation_params: parameters for new graph generation
-    :param metrics: quality metrics
-    :param parameters: parameters of graph optimiser
-    :param log: optional parameter for log object
-    :param archive_type: type of archive with best individuals
+    Multi-objective evolutionary graph optimiser named GPComp
     """
 
     def __init__(self, initial_graph: Union[Any, List[Any]], requirements,
                  graph_generation_params: 'GraphGenerationParams',
                  metrics: List[MetricsEnum],
                  parameters: Optional[GPGraphOptimiserParameters] = None,
-                 log: Log = None, archive_type=None):
+                 log: Log = None):
 
-        if not log:
-            self.log = default_log(__name__)
-        else:
-            self.log = log
+        super().__init__(initial_graph, requirements, graph_generation_params, metrics, parameters, log)
 
         self.graph_generation_params = graph_generation_params
         self.requirements = requirements
 
         self.parameters = GPGraphOptimiserParameters() if parameters is None else parameters
         self.parameters.set_default_params()
-        self.archive = archive_type if archive_type else SimpleArchive()
+        if isinstance(self.parameters.archive_type, ParetoFront):
+            self.archive = self.parameters.archive_type
+        else:
+            self.archive = SimpleArchive()
 
         self.max_depth = self.requirements.start_depth \
             if self.parameters.with_auto_depth_configuration and self.requirements.start_depth \
@@ -132,41 +127,50 @@ class GPGraphOptimiser:
 
         self.population = None
         self.initial_graph = initial_graph
-        self.history = OptHistory(metrics, parameters.history_folder)
-        self.history.clean_results()
 
-    def _create_randomized_pop_from_inital_pipeline(self, initial_pipeline) -> List[Individual]:
+    def _create_randomized_pop_from_inital_graph(self, initial_graphs: List[OptGraph]) -> List[Individual]:
         """
-        Fill first population with mutated variants of the initial_pipeline
-        :param initial_pipeline: Initial assumption for first population
+        Fill first population with mutated variants of the initial_graphs
+        :param initial_graphs: Initial assumption for first population
         :return: list of individuals
         """
         initial_req = deepcopy(self.requirements)
         initial_req.mutation_prob = 1
-        randomized_pop = ([mutation(types=self.parameters.mutation_types,
-                                    params=self.graph_generation_params,
-                                    ind=Individual(deepcopy(initial_pipeline)), requirements=initial_req,
-                                    max_depth=self.max_depth, log=self.log,
-                                    add_to_history=False)
-                           for _ in range(self.requirements.pop_size)])
+        randomized_pop = []
+        n_iter = self.requirements.pop_size * 10
+        while n_iter > 0:
+            initial_graph = np.random.choice(initial_graphs)
+            n_iter -= 1
+            new_ind = mutation(types=self.parameters.mutation_types,
+                               params=self.graph_generation_params,
+                               ind=Individual(deepcopy(initial_graph)),
+                               requirements=initial_req,
+                               max_depth=self.max_depth, log=self.log,
+                               add_to_history=False)
+            if new_ind not in randomized_pop:
+                # to suppress duplicated
+                randomized_pop.append(new_ind)
+
+            if len(randomized_pop) == self.requirements.pop_size - len(initial_graphs):
+                break
+
+        # add initial graph to population
+        for initial_graph in initial_graphs:
+            randomized_pop.append(Individual(deepcopy(initial_graph)))
+
         return randomized_pop
 
     def _init_population(self):
         if self.initial_graph:
-            if type(self.initial_graph) != list:
-                initial_graph = self.graph_generation_params.adapter.adapt(self.initial_graph)
-                self.population = self._create_randomized_pop_from_inital_pipeline(initial_graph)
-            else:
-                self.population = \
-                    [Individual(graph=self.graph_generation_params.adapter.adapt(o))
-                     for o in self.initial_graph]
-
+            adapted_graphs = [self.graph_generation_params.adapter.adapt(g) for g in self.initial_graph]
+            self.population = self._create_randomized_pop_from_inital_graph(adapted_graphs)
         if self.population is None:
             self.population = self._make_population(self.requirements.pop_size)
         return self.population
 
     def optimise(self, objective_function, offspring_rate: float = 0.5,
-                 on_next_iteration_callback: Optional[Callable] = None):
+                 on_next_iteration_callback: Optional[Callable] = None,
+                 show_progress: bool = True) -> Union[OptGraph, List[OptGraph]]:
         if on_next_iteration_callback is None:
             on_next_iteration_callback = self.default_on_next_iteration_callback
 
@@ -175,6 +179,9 @@ class GPGraphOptimiser:
         num_of_new_individuals = self.offspring_size(offspring_rate)
 
         with OptimisationTimer(log=self.log, timeout=self.requirements.timeout) as t:
+            pbar = tqdm(total=self.requirements.num_of_generations,
+                        desc="Generations", unit='gen', initial=1) if show_progress else None
+
             self.population = self._evaluate_individuals(self.population, objective_function, timer=t)
 
             if self.archive is not None:
@@ -184,8 +191,11 @@ class GPGraphOptimiser:
 
             self.log_info_about_best()
 
-            while t.is_time_limit_reached(self.generation_num) is False \
-                    and self.generation_num != self.requirements.num_of_generations - 1:
+            while (t.is_time_limit_reached(self.generation_num) is False
+                   and self.generation_num != self.requirements.num_of_generations - 1):
+
+                if self._is_stopping_criteria_triggered():
+                    break
 
                 self.log.info(f'Generation num: {self.generation_num}')
 
@@ -245,20 +255,21 @@ class GPGraphOptimiser:
                     self.archive.clear()
 
                 clean_operators_history(self.population)
+
+                if pbar:
+                    pbar.update(1)
+
+            if pbar:
+                pbar.close()
+
             best = self.result_individual()
             self.log.info('Result:')
             self.log_info_about_best()
 
-        output = [self.graph_generation_params.adapter.restore(ind.graph) for ind in best] if isinstance(best, list) \
-            else self.graph_generation_params.adapter.restore(best.graph)
+        output = [ind.graph for ind in best] if isinstance(best, list) else best.graph
 
         return output
 
-    def _convert_inds_to_external_result(self, individuals):
-        return [self.graph_generation_params.adapter.restore(ind.graph) for ind in individuals] \
-            if isinstance(individuals, list) \
-            else self.graph_generation_params.adapter.restore(individuals.graph)
-
     @property
     def best_individual(self) -> Any:
         if self.parameters.multi_objective:
@@ -303,15 +314,16 @@ class GPGraphOptimiser:
             self.max_depth += 1
 
     def get_best_individual(self, individuals: List[Any], equivalents_from_current_pop=True) -> Any:
-        best_ind = min(individuals, key=lambda ind: ind.fitness)
+        inds_to_analyze = [ind for ind in individuals if ind.fitness is not None]
+        best_ind = min(inds_to_analyze, key=lambda ind: ind.fitness)
         if equivalents_from_current_pop:
             equivalents = self.simpler_equivalents_of_best_ind(best_ind)
         else:
-            equivalents = self.simpler_equivalents_of_best_ind(best_ind, individuals)
+            equivalents = self.simpler_equivalents_of_best_ind(best_ind, inds_to_analyze)
 
         if equivalents:
             best_candidate_id = min(equivalents, key=equivalents.get)
-            best_ind = individuals[best_candidate_id]
+            best_ind = inds_to_analyze[best_candidate_id]
         return best_ind
 
     def simpler_equivalents_of_best_ind(self, best_ind: Any, inds: List[Any] = None) -> dict:
@@ -370,14 +382,8 @@ class GPGraphOptimiser:
             num_of_new_individuals = self.requirements.pop_size
         return num_of_new_individuals
 
-    def is_equal_fitness(self, first_fitness, second_fitness, atol=1e-10, rtol=1e-10):
-        return np.isclose(first_fitness, second_fitness, atol=atol, rtol=rtol)
-
     def default_on_next_iteration_callback(self, individuals, archive):
         try:
-            for individual in individuals:
-                individual.graph = \
-                    self.graph_generation_params.adapter.restore(individual.graph)
             self.history.add_to_history(individuals)
             self.history.save_current_results()
             archive = deepcopy(archive)
@@ -398,32 +404,13 @@ class GPGraphOptimiser:
                                                      objective_function=objective_function,
                                                      graph_generation_params=self.graph_generation_params,
                                                      timer=timer, is_multi_objective=self.parameters.multi_objective)
-        individuals_set = correct_if_population_has_nans(evaluated_individuals, self.log)
+        individuals_set = correct_if_has_nans(evaluated_individuals, self.log)
         return individuals_set
 
-
-@dataclass
-class GraphGenerationParams:
-    """
-    This dataclass is for defining the parameters using in graph generation process
-
-    :param adapter: the function for processing of external object that should be optimized
-    :param rules_for_constraint: set of constraints
-    """
-    adapter: BaseOptimizationAdapter = DirectAdapter()
-    rules_for_constraint: Optional[List[Callable]] = None
-    advisor: Optional[DefaultChangeAdvisor] = DefaultChangeAdvisor()
-
-
-def correct_if_population_has_nans(population, log):
-    len_before = len(population)
-    population = [ind for ind in population if ind.fitness is not None]
-    len_after = len(population)
-
-    if len_after != len_before:
-        log.info(f"None from individual's fitness were removed")
-
-    if len(population) == 0:
-        raise ValueError('All evaluations of fitness was unsuccessful.')
-
-    return population
+    def _is_stopping_criteria_triggered(self):
+        is_stopping_needed = self.stopping_after_n_generation is not None
+        if is_stopping_needed and self.num_of_gens_without_improvements == self.stopping_after_n_generation:
+            self.log.info(f'GP_Optimiser: Early stopping criteria was triggered and composing finished')
+            return True
+        else:
+            return False
