@@ -1,5 +1,5 @@
 from copy import deepcopy
-from typing import List, Optional, Union, Any
+from typing import Any, List, Optional, Union
 from uuid import uuid4
 
 from fedot.core.dag.graph_node import GraphNode
@@ -28,11 +28,11 @@ class OptNode:
     """
     Class for node definition in optimization graph (OptGraph)
 
-    :param content: alias of the content in node
-    :param nodes_from: parent nodes
+    :param content: content in node (name only or dict with name and params)
+    :param nodes_from: parent nodes in directed graph
     """
 
-    def __init__(self, content: dict,
+    def __init__(self, content: Union[str, dict],
                  nodes_from: Optional[List['OptNode']] = None,
                  log: Optional[Log] = None
                  ):
@@ -44,6 +44,9 @@ class OptNode:
         else:
             self.log = log
 
+        if isinstance(content, str):
+            content = {'name': content}
+
         self.nodes_from = nodes_from if nodes_from is not None else []
         self.content = {**content, **default_dict}
         self._operator = NodeOperator(self)
@@ -89,7 +92,8 @@ class OptGraph:
 
         self.uid = str(uuid4())
         self.nodes = []
-        self.operator = GraphOperator(self)
+        self.operator = GraphOperator(self, self._empty_postproc)
+        self._serialization_id = uuid4().hex
 
         if nodes:
             if isinstance(nodes, list):
@@ -98,6 +102,9 @@ class OptGraph:
             else:
                 self.add_node(nodes)
 
+    def _empty_postproc(self, nodes=None):
+        pass
+
     @property
     def _node_adapter(self):
         return NodeOperatorAdapter()
