@@ -1,16 +1,23 @@
 import csv
 import datetime
 import itertools
+import json
 import os
 import shutil
 from copy import deepcopy
 from dataclasses import dataclass
-from typing import (Any, List, Optional)
+from pathlib import Path
+from typing import TYPE_CHECKING, Any, Callable, List, Optional, Union
 from uuid import uuid4
 
+from fedot.core.optimisers.adapters import PipelineAdapter
+from fedot.core.serializers import Serializer
+
+if TYPE_CHECKING:
+    from fedot.core.optimisers.gp_comp.individual import Individual
+
 from fedot.core.optimisers.utils.multi_objective_fitness import MultiObjFitness
 from fedot.core.optimisers.utils.population_utils import get_metric_position
-from fedot.core.pipelines.template import PipelineTemplate
 from fedot.core.repository.quality_metrics_repository import QualityMetricsEnum
 from fedot.core.utils import default_fedot_data_dir
 
@@ -19,7 +26,7 @@ from fedot.core.utils import default_fedot_data_dir
 class ParentOperator:
     operator_name: str
     operator_type: str
-    parent_objects: List[PipelineTemplate]
+    parent_objects: List['Individual']
     uid: str = None
 
     def __post_init__(self):
@@ -32,71 +39,38 @@ class OptHistory:
     Contain history, convert Pipeline to PipelineTemplate, save history to csv
     """
 
-    def __init__(self, metrics=None, save_folder=None):
+    def __init__(self, metrics: List[Callable[..., float]] = None, save_folder=None):
         self.metrics = metrics
-        self.individuals = []
-        self.archive_history = []
-        self.pipelines_comp_time_history = []
-        self.archive_comp_time_history = []
-        self.parent_operators = []
-        self.save_folder = save_folder if save_folder \
+        self.individuals: List[List['Individual']] = []
+        self.archive_history: List[List['Individual']] = []
+        self.save_folder: str = save_folder if save_folder \
             else f'composing_history_{datetime.datetime.now().timestamp()}'
 
-    def _convert_pipeline_to_template(self, pipeline):
-        pipeline_template = PipelineTemplate(pipeline)
-        return pipeline_template
+    def add_to_history(self, individuals: List['Individual']):
+        self.individuals.append([deepcopy(ind) for ind in individuals])
 
-    def add_to_history(self, individuals: List[Any]):
-        new_individuals = []
-        pipelines_comp_time = []
-        parent_operators = []
-        try:
-            for ind in individuals:
-                pipeline = ind.graph  # was restored outside
-                new_ind = deepcopy(ind)
-                new_ind.graph = self._convert_pipeline_to_template(pipeline)
-                new_individuals.append(new_ind)
-                if hasattr(pipeline, 'computation_time'):
-                    pipelines_comp_time.append(pipeline.computation_time)
-                else:
-                    pipelines_comp_time.append(-1)
-                parent_operators.append(ind.parent_operators)
-            self.individuals.append(new_individuals)
-            self.pipelines_comp_time_history.append(pipelines_comp_time)
-        except Exception as ex:
-            print(f'Cannot add to history: {ex}')
-
-        self.parent_operators.append(parent_operators)
-
-    def add_to_archive_history(self, individuals: List[Any]):
-        try:
-            new_individuals = []
-            archive_comp_time = []
-            for ind in individuals:
-                new_ind = deepcopy(ind)
-                new_ind.graph = self._convert_pipeline_to_template(ind.graph)
-                new_individuals.append(new_ind)
-                archive_comp_time.append(ind.graph.computation_time)
-            self.archive_history.append(new_individuals)
-            self.archive_comp_time_history.append(archive_comp_time)
-        except Exception as ex:
-            print(f'Cannot add to archive history: {ex}')
+    def add_to_archive_history(self, individuals: List['Individual']):
+        self.archive_history.append([ind for ind in individuals])
 
     def write_composer_history_to_csv(self, file='history.csv'):
-        history_dir = os.path.join(default_fedot_data_dir(), self.save_folder)
+        history_dir = self._get_save_path()
         file = os.path.join(history_dir, file)
         if not os.path.isdir(history_dir):
             os.mkdir(history_dir)
         self._write_header_to_csv(file)
         idx = 0
+        adapter = PipelineAdapter()
         for gen_num, gen_inds in enumerate(self.individuals):
             for ind_num, ind in enumerate(gen_inds):
                 if self.is_multi_objective:
                     fitness = ind.fitness.values
                 else:
                     fitness = ind.fitness
-                row = [idx, gen_num, fitness, len(ind.graph.operation_templates), ind.graph.depth,
-                       self.pipelines_comp_time_history[gen_num][ind_num]]
+                ind_pipeline_template = adapter.restore_as_template(ind.graph, ind.computation_time)
+                row = [
+                    idx, gen_num, fitness,
+                    len(ind_pipeline_template.operation_templates), ind_pipeline_template.depth, ind.computation_time
+                ]
                 self._add_history_to_csv(file, row)
                 idx += 1
 
@@ -116,21 +90,36 @@ class OptHistory:
 
     def save_current_results(self, path: Optional[str] = None):
         if not path:
-            path = os.path.join(default_fedot_data_dir(), self.save_folder)
+            path = self._get_save_path()
         try:
             last_gen_id = len(self.individuals) - 1
             last_gen = self.individuals[last_gen_id]
             for ind_id, individual in enumerate(last_gen):
                 # TODO support multi-objective case
-                ind_path = os.path.join(path, str(last_gen_id), str(individual.graph.unique_pipeline_id))
+                ind_path = os.path.join(path, str(last_gen_id), str(individual.graph.uid))
                 additional_info = \
                     {'fitness_name': self.short_metrics_names[0],
                      'fitness_value': self.historical_fitness[last_gen_id][ind_id]}
-                individual.graph.export_pipeline(path=ind_path, additional_info=additional_info,
-                                                 datetime_in_path=False)
+                PipelineAdapter().restore_as_template(
+                    individual.graph, individual.computation_time
+                ).export_pipeline(path=ind_path, additional_info=additional_info, datetime_in_path=False)
         except Exception as ex:
             print(ex)
 
+    def save(self, json_file_path: os.PathLike = None) -> Optional[str]:
+        if json_file_path is None:
+            return json.dumps(self, indent=4, cls=Serializer)
+        with open(json_file_path, mode='w') as json_fp:
+            json.dump(self, json_fp, indent=4, cls=Serializer)
+
+    @staticmethod
+    def load(json_str_or_file_path: Union[str, os.PathLike] = None) -> 'OptHistory':
+        try:
+            return json.loads(json_str_or_file_path, cls=Serializer)
+        except json.JSONDecodeError as exc:
+            with open(json_str_or_file_path, mode='r') as json_fp:
+                return json.load(json_fp, cls=Serializer)
+
     def clean_results(self, path: Optional[str] = None):
         if not path:
             path = os.path.join(default_fedot_data_dir(), self.save_folder)
@@ -191,8 +180,22 @@ class OptHistory:
 
     @property
     def historical_pipelines(self):
-        return [ind.graph for ind in list(itertools.chain(*self.individuals))]
+        adapter = PipelineAdapter()
+        return [
+            adapter.restore_as_template(ind.graph, ind.computation_time)
+            for ind in list(itertools.chain(*self.individuals))
+        ]
 
     @property
     def is_multi_objective(self):
         return type(self.individuals[0][0].fitness) is MultiObjFitness
+
+    def _get_save_path(self):
+        if os.path.sep in self.save_folder:
+            # Defined path is full - there is no need to use default dir
+            # Create folder if it's not exists
+            if os.path.isdir(self.save_folder) is False:
+                os.makedirs(self.save_folder)
+            return self.save_folder
+        else:
+            return os.path.join(default_fedot_data_dir(), self.save_folder)
