@@ -6,6 +6,7 @@ from fedot.core.log import Log, default_log
 from fedot.core.operations.factory import OperationFactory
 from fedot.core.operations.operation import Operation
 from fedot.core.repository.default_params_repository import DefaultOperationParamsRepository
+from fedot.core.repository.operation_types_repository import OperationTypesRepository
 from fedot.core.utils import DEFAULT_PARAMS_STUB
 
 
@@ -90,7 +91,6 @@ class Node(GraphNode):
     def _filter_params(self, returned_params: Union[dict, tuple]) -> dict:
         """
         Filters out the desired parameter values from what Implementation returns
-
         :param returned_params: dictionary with
         """
         if isinstance(returned_params, tuple):
@@ -156,7 +156,6 @@ class Node(GraphNode):
 
         :param input_data: data used for operation training
         """
-
         if self.fitted_operation is None:
             self.fitted_operation, operation_predict = self.operation.fit(params=self.content['params'],
                                                                           data=input_data,
@@ -164,12 +163,13 @@ class Node(GraphNode):
         else:
             operation_predict = self.operation.predict(fitted_operation=self.fitted_operation,
                                                        data=input_data,
-                                                       is_fit_pipeline_stage=True)
+                                                       is_fit_pipeline_stage=True,
+                                                       params=self.content['params'])
 
         # Update parameters after operation fitting (they can be corrected)
-        not_source_node = 'source' not in self.operation.operation_type
         not_atomized_operation = 'atomized' not in self.operation.operation_type
-        if not_source_node and not_atomized_operation:
+
+        if not_atomized_operation and 'correct_params' in self.operation.metadata.tags:
             self.update_params()
         return operation_predict
 
@@ -181,6 +181,7 @@ class Node(GraphNode):
         :param output_mode: desired output for operations (e.g. labels, probs, full_probs)
         """
         operation_predict = self.operation.predict(fitted_operation=self.fitted_operation,
+                                                   params=self.content['params'],
                                                    data=input_data,
                                                    output_mode=output_mode,
                                                    is_fit_pipeline_stage=False)
@@ -198,11 +199,25 @@ class Node(GraphNode):
                 default_params = get_default_params(self.operation.operation_type)
                 if default_params is not None:
                     params = {**default_params, **params}
+                # take nested composer params if they appeared
+                if 'nested_space' in params:
+                    params = params['nested_space']
             self.content.update({'params': params})
 
     def __str__(self):
         return str(self.operation.operation_type)
 
+    @property
+    def tags(self):
+        """ Return tags of operation in the node. """
+        if 'atomized' in self.operation.operation_type:
+            # There are no tags for atomized operation
+            return []
+
+        info = OperationTypesRepository(operation_type='all').operation_info_by_id(self.operation.operation_type)
+        if info is not None:
+            return info.tags
+
 
 class PrimaryNode(Node):
     """
