@@ -2,18 +2,31 @@ import json
 import os
 from collections import Counter
 from datetime import datetime
-from typing import List, Optional, Tuple, Union
+from io import BytesIO
+from typing import TYPE_CHECKING, Callable, List, Optional, Tuple, Union
 from uuid import uuid4
 
 import joblib
+import numpy as np
 
 from fedot.core.log import Log, default_log
 from fedot.core.operations.atomized_template import AtomizedModelTemplate
-from fedot.core.operations.operation_template import OperationTemplate
+from fedot.core.operations.operation_template import OperationTemplate, check_existing_path
 from fedot.core.pipelines.node import Node, PrimaryNode, SecondaryNode
+
+if TYPE_CHECKING:
+    from fedot.core.pipelines.pipeline import Pipeline
+
 from fedot.core.repository.operation_types_repository import atomized_model_type
 
 
+class NumpyIntEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, np.integer):
+            return int(obj)
+        return super().default(self, obj)
+
+
 class PipelineTemplate:
     """
     Pipeline wrapper with 'export_pipeline'/'import_pipeline' methods
@@ -23,12 +36,21 @@ class PipelineTemplate:
     :params log: Log object to record messages
     """
 
-    def __init__(self, pipeline=None, log: Log = None):
+    def __init__(self, pipeline: 'Pipeline' = None, log: Log = None):
         self.total_pipeline_operations = Counter()
-        self.depth = pipeline.depth
-        self.operation_templates = []
-        self.unique_pipeline_id = str(uuid4()) if not pipeline.uid else pipeline.uid
-        self.struct_id = pipeline.root_node.descriptive_id if pipeline.root_node else ''
+        self.operation_templates: List[OperationTemplate] = []
+        if pipeline is not None:
+            self.depth = pipeline.depth
+            self.unique_pipeline_id = str(uuid4()) if not pipeline.uid else pipeline.uid
+            self.struct_id = pipeline.root_node.descriptive_id if pipeline.root_node else ''
+
+            # Save preprocessing operations
+            self.data_preprocessor = pipeline.preprocessor
+        else:
+            self.depth = 0
+            self.unique_pipeline_id = str(uuid4())
+            self.struct_id = ''
+            self.data_preprocessor = None
 
         try:
             self.computation_time = pipeline.computation_time
@@ -107,7 +129,7 @@ class PipelineTemplate:
                     else:
                         pipeline_template_dict['fitted_operation_path'] = None
 
-        json_data = json.dumps(pipeline_template_dict, indent=4)
+        json_data = json.dumps(pipeline_template_dict, indent=4, cls=NumpyIntEncoder)
 
         if path is None:
             return json_data, fitted_ops
@@ -122,20 +144,32 @@ class PipelineTemplate:
             pipeline_template_dict['additional_info'] = additional_info
 
         with open(os.path.join(absolute_path, f'{self.unique_pipeline_id}.json'), 'w', encoding='utf-8') as f:
-            f.write(json.dumps(pipeline_template_dict, indent='\t'))
+            f.write(json.dumps(pipeline_template_dict, indent='\t', cls=NumpyIntEncoder))
             resulted_path = os.path.join(absolute_path, f'{self.unique_pipeline_id}.json')
-            self.log.message(f'The pipeline saved in the path: {resulted_path}.')
+            self.log.debug(f'The pipeline saved in the path: {resulted_path}.')
 
         dict_fitted_operations = self._create_fitted_operations(absolute_path)
 
         return json_data, dict_fitted_operations
 
     def convert_to_dict(self, root_node: Node = None) -> dict:
+        """ Generate pipeline description in a form of dictionary """
+
         json_nodes = list(map(lambda op_template: op_template.convert_to_dict(), self.operation_templates))
+        for node in json_nodes:
+            if 'custom_params' in node and isinstance(node['custom_params'], dict):
+                for key in node['custom_params']:
+                    if isinstance(node['custom_params'][key], Callable):
+                        node['custom_params'][key] = None
+
+        # Store information about preprocessing
+        preprocessing_path = os.path.join('preprocessing', 'data_preprocessor.pkl')
+
         json_object = {
             "total_pipeline_operations": list(self.total_pipeline_operations),
             "depth": self.depth,
             "nodes": json_nodes,
+            "preprocessing": preprocessing_path
         }
         if root_node:
             json_object['descriptive_id'] = root_node.descriptive_id
@@ -143,6 +177,7 @@ class PipelineTemplate:
         return json_object
 
     def _create_fitted_operations(self, path=None):
+        """ Create .pkl files for operations using absolute path """
         dict_fitted_operations = {}
         for operation in self.operation_templates:
             dict_fitted_operations[f'operation_{operation.operation_id}'] = operation.export_operation(path)
@@ -150,6 +185,8 @@ class PipelineTemplate:
         if all([val is None for val in dict_fitted_operations.values()]):
             return None
 
+        # Save preprocessing module
+        dict_fitted_operations['preprocessing'] = self.export_preprocessing(path)
         return dict_fitted_operations
 
     def _prepare_paths(self, path: str, with_time: bool = True):
@@ -181,10 +218,10 @@ class PipelineTemplate:
 
             with open(path) as json_file:
                 json_object_pipeline = json.load(json_file)
-                self.log.message(f'The pipeline was imported from the path: {path}.')
+                self.log.debug(f'The pipeline was imported from the path: {path}.')
         else:
             json_object_pipeline = source
-            self.log.message(f'The pipeline was imported from dict.')
+            self.log.debug(f'The pipeline was imported from dict.')
 
         self._extract_operations(json_object_pipeline, path)
         self.convert_to_pipeline(self.link_to_empty_pipeline, path, dict_fitted_operations)
@@ -197,13 +234,14 @@ class PipelineTemplate:
         if os.path.isfile(absolute_path):
             self.unique_pipeline_id = os.path.splitext(name_of_file)[0]
         else:
-            message = f"The path to load a pipeline is not correct: {absolute_path}."
+            message = f'The path to load a pipeline is not correct: {absolute_path}.'
             self.log.error(message)
             raise FileNotFoundError(message)
 
     def _extract_operations(self, pipeline_json, path):
         operation_objects = pipeline_json['nodes']
 
+        # Update info about fitted operation
         for operation_object in operation_objects:
             if operation_object['operation_type'] == atomized_model_type():
                 filename = operation_object['atomized_model_json_path'] + '.json'
@@ -226,6 +264,19 @@ class PipelineTemplate:
         pipeline.nodes.clear()
         pipeline.add_node(root_node)
 
+        if path is not None and 'preprocessing' in os.listdir(path):
+            # Load data preprocessor and store it into the
+            path_to_preprocessor = os.path.join(path, 'preprocessing', 'data_preprocessor.pkl')
+            if os.path.exists(path_to_preprocessor):
+                restored_data_preprocessor = joblib.load(path_to_preprocessor)
+                pipeline.preprocessor = restored_data_preprocessor
+        elif dict_fitted_operations is not None and 'preprocessing' in dict_fitted_operations:
+            tmp_path = 'preprocessing.tmp'
+            with open(tmp_path, 'wb') as f:
+                f.write(BytesIO(dict_fitted_operations['preprocessing']).getbuffer())
+            pipeline.preprocessor = joblib.load(tmp_path)
+            os.remove(tmp_path)
+
     def roll_pipeline_structure(self, operation_object: ['OperationTemplate',
                                                          'AtomizedModelTemplate'],
                                 visited_nodes: dict, path: str = None, dict_fitted_operations: dict = None):
@@ -253,20 +304,31 @@ class PipelineTemplate:
                 node = SecondaryNode(operation_object.operation_type)
             else:
                 node = PrimaryNode(operation_object.operation_type)
+
             node.custom_params = operation_object.custom_params
+
             node.rating = operation_object.rating
 
         if hasattr(operation_object,
                    'fitted_operation_path') and operation_object.fitted_operation_path and path is not None:
             path_to_operation = os.path.join(path, operation_object.fitted_operation_path)
-            if not os.path.isfile(path_to_operation):
+
+            if "h2o" in operation_object.operation_type:
+                fitted_operation = load_h2o(path, self.log)
+
+            elif not os.path.isfile(path_to_operation):
                 message = f"Fitted operation on the path: {path_to_operation} does not exist."
                 self.log.error(message)
                 raise FileNotFoundError(message)
-
-            fitted_operation = joblib.load(path_to_operation)
+            else:
+                fitted_operation = joblib.load(path_to_operation)
         elif dict_fitted_operations is not None:
-            fitted_operation = joblib.load(dict_fitted_operations[f'operation_{operation_object.operation_id}'])
+            if "h2o" in operation_object.operation_type:
+                message = f"Loading h2o models from dict is not supported"
+                self.log.error(message)
+                raise TypeError(message)
+            else:
+                fitted_operation = joblib.load(dict_fitted_operations[f'operation_{operation_object.operation_id}'])
 
         operation_object.fitted_operation = fitted_operation
         node.fitted_operation = fitted_operation
@@ -280,6 +342,23 @@ class PipelineTemplate:
         visited_nodes[operation_object.operation_id] = node
         return node
 
+    def export_preprocessing(self, path: str = None):
+        """ Save preprocessing operations in pkl files and store full paths into dictionary """
+        if path:
+            path_fitted_preprocessing = os.path.join(path, 'preprocessing')
+            check_existing_path(path_fitted_preprocessing)
+
+            data_preprocessor_path = os.path.join(path_fitted_preprocessing, 'data_preprocessor.pkl')
+            if self.data_preprocessor is not None:
+                joblib.dump(self.data_preprocessor, data_preprocessor_path)
+                return data_preprocessor_path
+        else:
+            # dictionary with bytes of fitted operations
+            if self.data_preprocessor:
+                bytes_container = BytesIO()
+                joblib.dump(self.data_preprocessor, bytes_container)
+                return bytes_container
+
 
 def _is_nested_path(path):
     return path.find('nested') == -1
@@ -291,3 +370,13 @@ def extract_subtree_root(root_operation_id: int, pipeline_template: PipelineTemp
     root_node = pipeline_template.roll_pipeline_structure(root_node, {})
 
     return root_node
+
+
+def load_h2o(path_to_operation, log):
+    from fedot.core.operations.evaluation.automl import H2OSerializationWrapper
+    try:
+        return H2OSerializationWrapper.load_operation(path_to_operation)
+    except EnvironmentError as e:
+        message = f"Obtained type of H2O pipeline doesn't serializable"
+        log.error(message)
+        raise EnvironmentError(message)
