@@ -83,7 +83,8 @@ def in_sample_ts_forecast(pipeline, input_data: Union[InputData, MultiModalData]
     """
     Method allows to make in-sample forecasting. The actual values of the time
     series, rather than the previously predicted parts of the time series,
-    are used for forecasting next parts.
+    are used for forecasting next parts. Sources indices in input_data parameter
+    will be ignored
     time series ----------------|---|---|---|
     forecast                    |---|---|---|
 
@@ -96,6 +97,9 @@ def in_sample_ts_forecast(pipeline, input_data: Union[InputData, MultiModalData]
     task = input_data.task
     exception_if_not_ts_task(task)
 
+    # Is there need to fit on supplemented time-series
+    force_refit = is_force_refit_needed(pipeline)
+
     if isinstance(input_data, InputData):
         time_series = np.array(input_data.features)
         pre_history_ts = time_series[:-horizon]
@@ -138,6 +142,18 @@ def in_sample_ts_forecast(pipeline, input_data: Union[InputData, MultiModalData]
     # Make forecast iteratively moving throw the horizon
     final_forecast = []
     for _, border in zip(range(0, number_of_iterations), intervals):
+
+        # add fit by flag
+        if force_refit:
+            # change index for fitting
+            predict_idx = data.idx
+            data.idx = np.arange(0, len(data.features))
+            # change target for fitting
+            data.target = data.features
+            pipeline.fit_from_scratch(data)
+            # change index for predict
+            data.idx = predict_idx
+
         iter_predict = pipeline.predict(input_data=data)
         iter_predict = np.ravel(np.array(iter_predict.predict))
         final_forecast.append(iter_predict)
@@ -164,11 +180,12 @@ def in_sample_ts_forecast(pipeline, input_data: Union[InputData, MultiModalData]
     return final_forecast
 
 
-def fitted_values(train_predicted: OutputData, horizon_step: int = None) -> OutputData:
+def fitted_values(source_input: InputData, train_predicted: OutputData, horizon_step: int = None) -> OutputData:
     """ The method converts a multidimensional lagged array into an
     one-dimensional array - time series based on predicted values for training sample
 
-    :param train_predicted: OutputData
+    :param source_input: InputData which were used to train pipeline
+    :param train_predicted: OutputData with trained values
     :param horizon_step: index of elements for forecast. If None - perform
     averaging for all forecasting steps
     """
@@ -176,6 +193,9 @@ def fitted_values(train_predicted: OutputData, horizon_step: int = None) -> Outp
     if horizon_step is not None:
         # Take particular forecast step
         copied_data.predict = copied_data.predict[:, horizon_step]
+        if isinstance(copied_data.idx, list):
+            # if indices can not be incremented, replace it
+            copied_data.idx = generate_ids(source_input, copied_data, expand=False)
         copied_data.idx = copied_data.idx + horizon_step
         return copied_data
     else:
@@ -183,7 +203,12 @@ def fitted_values(train_predicted: OutputData, horizon_step: int = None) -> Outp
         forecast_length = copied_data.task.task_params.forecast_length
 
         # Extend source index range
-        indices_range = np.arange(copied_data.idx[0], copied_data.idx[-1] + forecast_length + 1)
+        if isinstance(copied_data.idx, list):
+            indices_range = generate_ids(source_input, copied_data, expand=True)
+        else:
+            # if indices can be incremented
+            indices_range = np.arange(copied_data.idx[0],
+                                      copied_data.idx[-1] + forecast_length)
 
         # Lagged matrix with indices in cells
         _, idx_matrix = ts_to_table(idx=indices_range,
@@ -193,7 +218,6 @@ def fitted_values(train_predicted: OutputData, horizon_step: int = None) -> Outp
 
         # For every index calculate mean predictions (by all forecast steps)
         final_predictions = []
-        indices_range = indices_range[:-1]
         for index in indices_range:
             vals = predicted_matrix[idx_matrix == index]
             mean_value = np.mean(vals)
@@ -203,7 +227,7 @@ def fitted_values(train_predicted: OutputData, horizon_step: int = None) -> Outp
         return copied_data
 
 
-def in_sample_fitted_values(train_predicted: OutputData) -> OutputData:
+def in_sample_fitted_values(source_input: InputData, train_predicted: OutputData) -> OutputData:
     """ Perform in sample validation based on training sample """
     forecast_length = train_predicted.task.task_params.forecast_length
     all_values = []
@@ -223,7 +247,11 @@ def in_sample_fitted_values(train_predicted: OutputData) -> OutputData:
     copied_data.predict = np.array(all_values)
     # Update indices
     first_id = copied_data.idx[0]
-    copied_data.idx = np.arange(first_id, first_id + len(all_values))
+    if isinstance(first_id, str):
+        indices_range = generate_ids(source_input, copied_data, expand=True)
+        copied_data.idx = indices_range[:-1]
+    else:
+        copied_data.idx = np.arange(first_id, first_id + len(all_values))
 
     return copied_data
 
@@ -287,3 +315,38 @@ def _calculate_intervals(last_index_pre_history, amount_of_iterations, scope_len
 def exception_if_not_ts_task(task):
     if task.task_type != TaskTypesEnum.ts_forecasting:
         raise ValueError(f'Method forecast is available only for time series forecasting task')
+
+
+def generate_ids(source_input, output_data, expand: bool):
+    """
+    Create new indices for fitted time series values. Process shift after
+    starting clipping.
+
+    :param source_input: source idx, features and target
+    :param output_data: OutputData after pipeline fit
+    :param expand: there is a need to take into account the forecasting horizon.
+    If True, expand indices.
+    """
+    forecast_len = source_input.task.task_params.forecast_length
+    source_idx = source_input.idx[:-forecast_len]
+
+    # Calculate difference between source idx len and new
+    clipped_starting_values = len(source_idx) - len(output_data.idx)
+
+    if expand:
+        indices_range = np.arange(clipped_starting_values + 1,
+                                  len(source_idx) + forecast_len + 1)
+    else:
+        indices_range = np.arange(clipped_starting_values + 1,
+                                  len(source_idx) + 1)
+    return indices_range
+
+
+def is_force_refit_needed(pipeline) -> bool:
+    """ Determine if force refit needed for current pipeline based on nodes tags """
+    is_needed = False
+    for node in pipeline.nodes:
+        if 'in_sample_refit' in node.tags:
+            return True
+
+    return is_needed
