@@ -1,9 +1,11 @@
 from abc import ABC, abstractmethod
 from typing import Callable, ClassVar
-from copy import deepcopy
+from copy import deepcopy, copy
 from datetime import timedelta
 
 import numpy as np
+from scipy.sparse import csr_matrix
+from sklearn.preprocessing import LabelEncoder
 
 from fedot.core.log import Log, default_log
 from fedot.core.repository.tasks import TaskTypesEnum
@@ -35,7 +37,7 @@ class HyperoptTuner(ABC):
         self.pipeline = pipeline
         self.task = task
         self.iterations = iterations
-        self.max_seconds = int(timeout.seconds)
+        self.max_seconds = int(timeout.seconds) if timeout is not None else None
         self.init_pipeline = None
         self.init_metric = None
         self.is_need_to_maximize = None
@@ -143,7 +145,8 @@ class HyperoptTuner(ABC):
 
         if self.is_need_to_maximize is True:
             # Maximization
-            init_metric = self.init_metric - deviation
+            init_metric = -1 * (self.init_metric - deviation)
+            obtained_metric = -1 * obtained_metric
             if obtained_metric >= init_metric:
                 self.log.info(f'{prefix_tuned_phrase} {obtained_metric:.3f} equal or '
                               f'bigger than initial (- 5% deviation) {init_metric:.3f}')
@@ -195,7 +198,7 @@ class HyperoptTuner(ABC):
             preds, test_target = cv_time_series_predictions(pipeline, data, log=self.log,
                                                             cv_folds=self.cv_folds,
                                                             validation_blocks=self.validation_blocks)
-        return test_target, preds
+        return preds, test_target
 
     @property
     def _default_metric_value(self):
@@ -205,6 +208,28 @@ class HyperoptTuner(ABC):
             return MAX_METRIC_VALUE
 
 
+def _create_multi_target_prediction(target, optimal=True):
+    """ Function creates an array of shape (target len, num classes)
+    with classes probabilities from target values, used in _greater_is_better
+
+    :param target: target for define what problem is solving (max or min)
+    :param optimal: whether return optimal probabilities or not
+
+    :return : 2d-array of classes probabilities
+    """
+
+    len_target = target.shape[0]
+
+    if optimal:
+        multi_target = csr_matrix((np.ones(len_target), (np.arange(len_target),
+                                                         target.reshape((len_target,))))).A
+    else:
+        multi_target = np.zeros((len_target, len(np.unique(target))))
+        multi_target[:, 0] = 1
+
+    return multi_target
+
+
 def _greater_is_better(target, loss_function, loss_params) -> bool:
     """ Function checks is metric (loss function) need to be minimized or
     maximized
@@ -216,15 +241,25 @@ def _greater_is_better(target, loss_function, loss_params) -> bool:
     :return : bool value is it good to maximize metric or not
     """
 
+    if isinstance(target[0], str):
+        # Target for classification contain string objects
+        le = LabelEncoder()
+        target = le.fit_transform(target)
+
     if loss_params is None:
-        metric = loss_function(target, target)
-    else:
-        try:
-            metric = loss_function(target, target, **loss_params)
-        except Exception:
-            # Multiclass classification task
-            metric = 1
-    if int(round(metric)) == 0:
-        return False
-    else:
+        loss_params = {}
+
+    try:
+        optimal_metric = loss_function(target, target, **loss_params)
+        not_optimal_metric = loss_function(target, np.zeros_like(target), **loss_params)
+    except Exception:
+        optimal_multi_target = _create_multi_target_prediction(target, True)
+        not_optimal_multi_target = _create_multi_target_prediction(target, False)
+
+        optimal_metric = loss_function(target, optimal_multi_target, **loss_params)
+        not_optimal_metric = loss_function(target, not_optimal_multi_target, **loss_params)
+
+    if optimal_metric > not_optimal_metric:
         return True
+    else:
+        return False
